__tool_name__ = 'hackingtools'
from .core import Connections as __Connections
from .core import Config, Utils, Logger, Repositories, Objects

__config__ = Config.getConfig(parentKey='core', key='import_modules')

import os as __os
from os import listdir as __listdir
from os.path import isfile as __isfile
from os.path import join as __join
import importlib as ____importlib
import types as __types
import inspect as __inspect
import shutil as __shutil
import ast as __ast
import json as __json
import sys as __sys
try:
    import readline as __readline
    __readline.parse_and_bind('tab: complete')
except:
    pass
from time import sleep as __sleep


# try:
#     from pip import main as pipmain
# except ImportError:
#     from pip._internal import main as pipmain

__modules_loaded__ = {}

# If we want to be en Pool, import its Functions
global __WANT_TO_BE_IN_POOL__
__WANT_TO_BE_IN_POOL__ = Config.getConfig(parentKey='core', key='__WANT_TO_BE_IN_POOL__')
if __WANT_TO_BE_IN_POOL__:
    from .core import Pool
else:
    Logger.printMessage('Pool not loaded', 'Change config or execute ht.Pool.switchPool() when you want it', is_warn=True)

# If it's Django, import it's Functions
__amidjango__ = Utils.amIdjango(__name__)
if __amidjango__:
    from .core import DjangoFunctions

__this_dir__ = __os.path.dirname(__os.path.abspath(__file__))

__blacklist_extensions__ = __config__['__blacklist_extensions__']
__blacklist_directories__ = __config__['__blacklist_directories__']
__ignore_files__ = __config__['__ignore_files__']
__ignore_folders__ = __config__['__ignore_folders__']
__class_name_starts_with_modules__ = __config__['__class_name_starts_with_modules__']
__function_name_starts_with_modules__ = __config__['__function_name_starts_with_modules__']
__default_class_name_for_all__ = __config__['__default_class_name_for_all__']

__default_template_modules_ht__ = __config__['__default_template_modules_ht__']

__cant_install_requirements__ = Config.getConfig(parentKey='core', key='__cant_install_requirements__')

__workers__ = []

__telegrambot_name__ = 'ht-bot'
__telegrambot_token__ = ''

global __is_all_loaded__
__is_all_loaded__ = False

def __checkLibraryUpdate__():
    try:
        # Check Version Update
        import pkg_resources as __pkg
        __version__ = __pkg.get_distribution(__tool_name__).version
        import requests as __req 
        from bs4 import BeautifulSoup as __bs4
        __pypipage__ = __req.get("https://pypi.org/project/hackingtools/")
        if __pypipage__.status_code == 200:
            __pageContent = __bs4(__pypipage__.content, 'html.parser')
            __pypiHeader = __pageContent.find('h1', {'class' : 'package-header__name'}).text
            __pypiVersion = __pypiHeader.replace(__tool_name__, '').replace('\n', ' ').replace('\r', '').replace(' ', '')
            if not __version__ == __pypiVersion:
                Logger.printMessage('Version outdated... Newer version available: {v}'.format(v=__pypiVersion), is_warn=True)
    except:
        pass

def getModulesNames():
    """Return's an Array with all the modules loaded names (ht_shodan, ht_nmap, etc.)

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    modules_names = []
    for tools in __modules_loaded__:
        modules_names.append(tools.split('.')[-1])
    modules_names.sort()
    return modules_names

def getModulesFromCategory(category):
    mods = []
    for m in getModulesNames():
        if getModuleCategory(m) == category:
            mods.append(m)
    return mods

def getCategories():
    data = []
    for mods in __modules_loaded__:
        if mods.split('.')[1] not in data:
            data.append(mods.split('.')[1])
    return data

def getModule(moduleName):
    """Return's and load's a module into a variable passing a module name as parameter

    Parameters
    ----------
        moduleName = String

    Return
    ----------
        eval(module)
    """
    #Logger.printMessage('Initiation of {moduleName}'.format(moduleName=moduleName), debug_module=True)

    # While the mods are loading with workers, wait por it
    while any( [ work.startswith('import-module-') for work in getWorkers() ] ):
        __sleep(0.1)

    while not __is_all_loaded__:
        print('loading modules yet, please wait')
        __sleep(0.3)

    for m in __modules_loaded__:
        if moduleName in m:
            if not 'ht_' in moduleName:
                moduleName = 'ht_{m}'.format(m=moduleName)
            sentence = 'modules.{category}.{mod}.{moduleName}.StartModule()'.format(category=m.split('.')[1], mod='_'.join(moduleName.split('_')[1:]), moduleName=moduleName)
            return eval(sentence)
    Logger.printMessage('Looks like {mod} is not loaded on HackingTools. Look the first import in log. You could have some error in your code :)'.format(mod=moduleName), is_error=True)

def getModules():
    data = []
    for mods in __modules_loaded__:
        data.append(
            'modules.{name}.{classInit}()'.format(name=mods.split('.')[-1], classInit=__default_class_name_for_all__))
    return data

def getModuleCategory(moduleName):
    for m in __modules_loaded__:
        if moduleName.replace('ht_', '') == m.split('.')[3].replace('ht_', ''):
            return m.split('.')[1]
    return None

def getFunctionsNamesFromModule(module_name):
    """Returns an Array with the functions of a module you choose

    Parameters
    ----------
        module_name = String

    Return
    ----------
        Array
    """
    try:
        for mod in __modules_loaded__:
            if module_name in mod and not isinstance(__modules_loaded__[mod], str):
                return list(__modules_loaded__[mod].keys())
            elif module_name in mod:
                return list(__modules_loaded__[mod])
        return []
    except:
        return []

def getWorkers():
    return __workers__

def getWorker(nameWorker):
    try:
        return Utils.getWorkers()[nameWorker]
    except:
        return None

def getWorkerLastResponse(nameWorker):
    try:
        return Utils.getWorkers()[nameWorker][0].getLastResponse()
    except:
        return ''

def setDebugCore(on=True):
    """Set Debug Log from Core to on/off

    Parameters
    ----------
        on = boolean (True/False)

    Return
    ----------
        None
    """
    Logger.setDebugCore(on)

def setDebugModule(on=True):
    """Set Debug Log from Modules to on/off

    Parameters
    ----------
        on = boolean (True/False)

    Return
    ----------
        None
    """
    Logger.setDebugModule(on)

def switchPool():
    global __WANT_TO_BE_IN_POOL__
    if __WANT_TO_BE_IN_POOL__:
        __WANT_TO_BE_IN_POOL__ = False
    else:
        __WANT_TO_BE_IN_POOL__ = True

def wantPool():
    global __WANT_TO_BE_IN_POOL__
    return __WANT_TO_BE_IN_POOL__

def worker(workerName, functionCall, args=(), timesleep=1, loop=True, run_until_ht_stops=False, log=False, timeout=None):
    Utils.startWorker(workerName, functionCall, args, int(timesleep), loop, run_until_ht_stops, log, timeout)
    if loop:
        __workers__.append(workerName)

def stopWorker(nameWorker):
    try:
        Utils.stopWorker(nameWorker)
    except Exception as e:
        Logger.printMessage(str(e), is_error=True)

def startTelegramBot():
    #setTelegramBotToken
    worker('telegram-bot', 'ht.core.Objects.TelegramBotCoreHT.run', loop=False, log=__amidjango__)

# === __getModulesJSON__ ===
def __getModulesJSON__():
    """
    Returns an Array with the modules loaded

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    conf = {}
    for mod in __modules_loaded__:
        if isinstance(__modules_loaded__[mod], str):
            conf[mod] = '1 function'
        else:
            conf[mod] = '{func} functions'.format(func=len(__modules_loaded__[mod]))
    # Logger.printMessage('Modules loaded as JSON automatically:', conf, debug_module=True)
    return __modules_loaded__

def __getModuleCalls__(module_name=None):
    """Return an Array with the modules calls you can write for importing directly a module.
    Also, it logs into your console the calls formated for well reading.

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    Logger.printMessage('Modules :', debug_module=True)
    if module_name:
        for mods in getModules():
            if module_name in mods:
                return mods
    return [mods for mods in getModules()]

def __getModulesFullConfig__():
    """Return's an Array with all the config of all modules loaded

    Parameters
    ----------
        None

    Return
    ----------
        Array
    """
    modules_conf = {}
    for module in getModulesNames():
        module_conf = Config.getConfig(parentKey='modules', key=module)
        if module_conf:
            modules_conf[module] = module_conf
    return modules_conf

def __getModuleConfig__(moduleName):
    """Return's an Array with the config of a module passed as parameter

    Parameters
    ----------
        moduleName = String

    Return
    ----------
        Array/None
    """
    if moduleName in getModulesNames():
        actualConf = __getModulesJSON__()
        for mod in actualConf:
            if moduleName in mod.split('.')[-1]:
                return actualConf[mod]
    return None

def __getModulesConfig__():
    return [{m: Config.getConfig(parentKey='modules', key=m.split('.')[-1])} for m in __modules_loaded__]

def __createModule__(moduleName, category):
    """
    Iniciamos con el comando anterior la instancia del modulo
    """
    Logger.printMessage('Creating {moduleName} on {category}'.format(moduleName=moduleName, category=category),
                        debug_module=True)
    moduleName = moduleName.replace(" ", "_").lower()
    category = category.lower()
    categories = getCategories()
    if category not in categories:
        __createCategory__(category)
    dir_actual = __os.path.dirname(__file__)
    if not __os.path.isdir(
            '{dir}/modules/{category}/{moduleName}'.format(dir=dir_actual, category=category, moduleName=moduleName)):
        __os.makedirs(
            '{dir}/modules/{category}/{moduleName}'.format(dir=dir_actual, category=category, moduleName=moduleName))
    if not __os.path.exists('{dir}/modules/{category}/__init__.py'.format(dir=dir_actual, category=category)):
        f = open('{dir}/modules/{category}/__init__.py'.format(dir=dir_actual, category=category), "w")
        f.write('')
    if not __os.path.exists(
            '{dir}/modules/{category}/{moduleName}/ht_{moduleName}.py'.format(dir=dir_actual, category=category,
                                                                              moduleName=moduleName)):
        f = open('{dir}/modules/{category}/{moduleName}/ht_{moduleName}.py'.format(dir=dir_actual, category=category,
                                                                                   moduleName=moduleName), "w")
        f.write(__default_template_modules_ht__.replace('{moduleName}', moduleName))
    if not __os.path.exists(
            '{dir}/modules/{category}/{moduleName}/__init__.py'.format(dir=dir_actual, category=category,
                                                                       moduleName=moduleName)):
        f = open('{dir}/modules/{category}/{moduleName}/__init__.py'.format(dir=dir_actual, category=category,
                                                                            moduleName=moduleName), "w")
        f.write('')
    if not __os.path.isdir('{dir}/core/config_modules_django/{category}/'.format(dir=dir_actual, category=category)):
        __os.mkdir('{dir}/core/config_modules_django/{category}/'.format(dir=dir_actual, category=category))
    if not __isfile(
            '{dir}/core/config_modules_django/{category}/{moduleName}.json'.format(dir=dir_actual, category=category,
                                                                                   moduleName=moduleName)):
        with open('{dir}/core/config_modules_django/{category}/ht_{moduleName}.json'.format(dir=dir_actual,
                                                                                            category=category,
                                                                                            moduleName=moduleName),
                  "w") as fp:
            fp.write(__json.dumps({}))
    # temp_path, hackingtools_dir = __os.path.split(dir_actual)
    # temp_path, library_dir = __os.path.split(temp_path)
    # urls_file = __os.path.join(temp_path, 'urls.py')
    # insert_url_django(urls_file, moduleName) # TODO edit urls for auto URLs when creating module
    # print("{msg}".format(msg=urls_file))
    # Reload variables on client side
    # global hackingtools
    # reload(hackingtools)
    Config.__createModuleTemplateConfig__(moduleName, category)
    __importModules__()
    return

def __removeModule__(moduleName, category):
    modulePath = __join(__this_dir__, 'modules', category, moduleName.replace('ht_', ''))
    try:
        __os.chmod(modulePath, 777)
    except:
        pass
    try:
        if __os.path.isdir(modulePath):
            __shutil.rmtree(modulePath, ignore_errors=True)
    except:
        pass
    try:
        if __os.path.isdir(modulePath):
            __os.rmdir(modulePath)
    except:
        pass
    try:
        if __os.path.isdir(modulePath):
            __os.removedirs(modulePath)
    except:
        pass

def __createCategory__(categoryName):
    categoryName = categoryName.lower()
    categories = getCategories()
    dir_actual = __os.path.dirname(__file__)
    if categoryName not in categories:
        if not __os.path.isdir('{dir}/modules/{category}/'.format(dir=dir_actual, category=categoryName)):
            __os.makedirs('{dir}/modules/{category}'.format(dir=dir_actual, category=categoryName))

def __cleanOutputModules__():
    for mod in getModulesNames():
        cat = getModuleCategory(mod)
        output_dir = __join(__os.path.dirname(__file__), 'modules', cat, mod.replace('ht_', ''), 'output')
        if __os.path.isdir(output_dir):
            for temp_file in __listdir(output_dir):
                __os.remove(__join(output_dir, temp_file))


# TODO Continue documentation here

# Import Modules

# Core method - Usado por: __importModules__()
def __listDirectory__(directory, files=False, exclude_pattern_starts_with=None):
    """
    Devuelve las carpetas contenidas en el directorio indicado. Si se quieren listar los 
    ficheros se deberá indicar el argumento files=True. En el caso de querer excluir ficheros o carpetas
    se indicará en el argumento exclude_pattern_starts_with con el comienzo de los mism__os.
    """
    mypath = __join(__this_dir__, directory)
    data = ''

    if files:
        data = [f for f in __listdir(mypath) if __isfile(__join(mypath, f))]
    else:
        data = [f for f in __listdir(mypath) if not __isfile(__join(mypath, f))]

    if __blacklist_extensions__:
        new_data = []
        for d in data:
            has_extension = False
            for black_ext in __blacklist_extensions__:
                if black_ext in d:
                    has_extension = True
            if not has_extension:
                new_data.append(d)
        data = new_data

    if exclude_pattern_starts_with:
        new_data = []
        for d in data:
            if not d.startswith(exclude_pattern_starts_with):
                new_data.append(d)
        data = new_data

    if __blacklist_directories__:
        new_data = []
        for d in data:
            for direc in __blacklist_directories__:
                if not direc in d:
                    new_data.append(d)
        data = new_data

    if __ignore_folders__:
        new_data = []
        for d in data:
            exists = False
            for direc in __ignore_folders__:
                if direc in d:
                    exists = True
            if not exists:
                new_data.append(d)
        data = new_data

    if files and __ignore_files__:
        new_data = []
        for d in data:
            for file_ign in __ignore_files__:
                if not file_ign in d:
                    new_data.append(d)
        data = new_data

    return data

# Core method - Usado por: __importModules__()
def __getModules__(directory='', exclude_pattern_starts_with='.'):
    """
    Devuelve las carpetas o ficheros contenidas en cada módulo del directorio actual, expluyendo de la raiz 
    los directorios que empiecen por punto. 
    """
    data = {}
    dirs = __listDirectory__(directory=directory, exclude_pattern_starts_with=exclude_pattern_starts_with)
    if dirs:
        for d in dirs:
            if not __isfile(d):
                sub_dirs = __listDirectory__(directory=d)  # Tengo que excluir el __pycache__
                data[d] = {}
                for sub in sub_dirs:  # Tipo de Herramientas (OSINT, SQLi, etc.)
                    sub_dirs_tools = __listDirectory__(directory='{d}/{sub}'.format(d=d, sub=sub))
                    data[d][sub] = {}
                    for tool in sub_dirs_tools:
                        sub_dirs_tool_files = __listDirectory__(directory='{d}/{sub}/{tool}'.format(d=d, sub=sub, tool=tool), files=True)
                        data[d][sub][tool] = sub_dirs_tool_files
                subdirectorio = __listDirectory__(directory=d, files=True)
                if len(subdirectorio) > 0:
                    data[d]['files'] = subdirectorio
            else:
                try:
                    data['files'].append(d)
                except:
                    data['files'] = []
                    data['files'].append(d)
    else:
        dirs_files = __listDirectory__(directory=directory, files=True)
        data[directory] = dirs_files
    return data

# Core method - Usado por: __importModules__()
def __methodsFromModule__(cls):
    """
    Devuelve los métodos que tiene una clase pasada como parametro
    """
    return [x for x in dir(getattr(cls, __default_class_name_for_all__)) if not x.startswith(__function_name_starts_with_modules__)]

# Core method - Usado por: __importModules__()
def __classNameFromModule__(cls):
    return [x for x in dir(cls) if __inspect.isclass(getattr(cls, x)) and x.startswith(__class_name_starts_with_modules__)]

def __importModule__(modules, category, moduleName):
    module_import_string = 'from .{modules}.{category}.{tool} import {toolFileName}'.format(modules=modules, category=category, tool=moduleName.replace('ht_', ''), toolFileName=moduleName)
    module_import_string_no_from = '{modules}.{category}.{tool}.{toolFileName}'.format(modules=modules, category=category, tool=moduleName.replace('ht_', ''), toolFileName=moduleName)
    try:
        exec(module_import_string)
        # globals()[module_name] = __importlib.import_module(module_import_string)
        module_className = __classNameFromModule__(eval(moduleName))
        module_functions = __methodsFromModule__(eval(moduleName))
        # Logger.printMessage(message='{mod} loaded'.format(mod=module_name), debug_module=True)
        if len(module_functions) > 0:
            __modules_loaded__[module_import_string_no_from] = {}
            for mod_func in module_functions:
                functionParams = Utils.getFunctionsParams(category=category, moduleName=moduleName, functionName=mod_func, i_want_list=True)
                original_params = Utils.getFunctionsParams(category=category, moduleName=moduleName, functionName=mod_func)

                __modules_loaded__[module_import_string_no_from][mod_func] = {}
                __modules_loaded__[module_import_string_no_from][mod_func]['params'] = functionParams if len(functionParams) > 0 else False
                __modules_loaded__[module_import_string_no_from][mod_func]['original_params'] = original_params if original_params else None
        else:
            __modules_loaded__[module_import_string_no_from] = 'Sin funciones...'
    except Exception as e:
        Logger.printMessage(str(e), is_error=True)
        new_module_name = str(e).split("'")[1]
        if 'No module named' in str(e):

            if not moduleName in __cant_install_requirements__:
                __cant_install_requirements__[moduleName] = []

            if not new_module_name in __cant_install_requirements__[moduleName]:

                try:
                    Logger.printMessage(message='__importModules__', description='Trying to install module {m}'.format(m=new_module_name), is_warn=True)
                    import subprocess
                    p = subprocess.Popen([__sys.executable, '-m', 'pip', 'install', new_module_name], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                    out = p.communicate()
                    if 'EnvironmentError' in out[0]:
                        Logger.printMessage(message='__importModules__', description='{moduleName} {error}'.format(moduleName=new_module_name, error='Could not install in environment'), is_error=True)
                    # pipmain([sys.executable, '-m', 'pip', 'install', '--user', new_module_name])
                except:
                    pass

                if not new_module_name in __cant_install_requirements__[moduleName]:
                    __cant_install_requirements__[moduleName].append(new_module_name)

                if not 'hackingtools' in new_module_name:
                    Config.add_requirements_ignore(moduleName, new_module_name)

                Logger.printMessage(message='__importModules__', description='{moduleName} {error}'.format(moduleName=module_import_string, error=str(e)), is_error=True)

# Core method
def __importModules__():
    """
    Método que busca dentro de las carpetas junto a este fichero (ignorando las directorios marcados anteriormente)
    y como subcarpetas tiene que haber el nombre del tipo de herramienta que es y debajo de esas carpetas
    tienen que estar los directorios individualmente por herramientas que se incorpore a la librería
    """
    Logger.printMessage(message='{meth}'.format(meth='__importModules__'), description='Loading modules...', debug_module=True)
    modules = __getModules__()
    for modu in modules:
        for submod in modules[modu]:
            for files in modules[modu][submod]:
                try:
                    module_name = modules[modu][submod][files][0].split(".")[0]
                    if not __amidjango__:
                        worker("import-module-{m}".format(m=module_name), 'ht.__importModule__', args=(modu, submod, module_name), loop=False, log=False) # Threaded
                    else:
                        __importModule__(modules=modu, category=submod, moduleName=module_name)
                except Exception as e:
                    Logger.printMessage(message='__importModules__', description='{moduleName} File not found: {error}'.format(moduleName=files, error=str(e)), is_error=True)
                    pass
    global __is_all_loaded__
    __is_all_loaded__ = True

worker('check-library-updated', 'ht.__checkLibraryUpdate__', loop=False, run_until_ht_stops=True, log=__amidjango__)
worker('refresh-pool-servers', 'ht.Pool.__checkPoolNodes__', timesleep=180, run_until_ht_stops=True, log=__amidjango__)
worker('clear-htpass-files', 'ht.Config.__cleanHtPassFiles__', timesleep=100, run_until_ht_stops=True, log=__amidjango__)
worker('clear-uploaded-modules-temp', 'ht.Repositories.clearUploadsTemp', timesleep=200, run_until_ht_stops=True, log=__amidjango__)
worker('repositories-get-online-servers', 'ht.Repositories.getOnlineServers', loop=False, run_until_ht_stops=True, log=__amidjango__)
worker('clear-output-modules', 'ht.__cleanOutputModules__', timesleep=200, run_until_ht_stops=True, log=__amidjango__)

if __telegrambot_token__:
    Config.setTelegramBotToken(__telegrambot_token__)
    startTelegramBot()

__importModules__()

# try:
#     for t in threads:
#         t.join()
# except:
#     pass


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os, socket

config = Config.getConfig(parentKey='modules', key='ht_bannergrabber')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = 'grabPortBanner'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_bannergrabber'), debug_module=True)

	def grabPortBanner(self, ip, port):
		s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		try:
			s.connect(( str(ip) , int(port) ))
			s.settimeout(1)
			banner = s.recv(1024)
			return banner.decode().strip()
		except:
			return ''


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os

import urllib.parse
import requests
from bs4 import BeautifulSoup

config = Config.getConfig(parentKey='modules', key='ht_router')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = 'getDefaultLogin'
		self._funcArgFromFunc_ = {
			'getDefaultLogin' : {
				'routerBrand' : {
					'router' : 'getBrands' 
				}
			}
		}
		self.brands = [ "2Wire, Inc.","3M","360 Systems","3COM","Accelerated Networks","ACCTON","Acer","Actiontec","Adaptec","ADC Kentrox", "AdComplete.com", "AddPac Technology", "Adobe", "ADT", "Adtech", "Adtran", "Advanced Integration", "AIRAYA Corp", "Airlink", "AirLink Plus", "Aironet", "Airway", "Aladdin", "Alcatel", "Alien Technology", "Allied Telesyn", "Allnet", "Allot", "Alteon", "Ambit", "AMI", "Amino", "AmpJuke", "Amptron", "AMX", "Apache", "Apache Project", "APC", "Apple", "Apple Computer", "Arris", "Arrowpoint", "Aruba", "Asante", "Ascend", "Ascom", "AST", "Asus", "AT&T", "Atlassian", "Attachmate", "Audioactive", "Autodesk", "Avaya", "Avenger News System (ANS)", "AWARD", "Axis", "Axis Communications", "Axway", "Barco, Inc.", "Barracuda", "Bay Networks", "Best Practical Solutions", "BestPractical", "Bewan", "Billion", "Bintec", "Biodata", "Biostar", "Biscom", "BizDesign", "Black Duck Software, Inc.", "Black Widow Web Design Ltd", "Blitzz Technologies", "BMC Software", "Bomgar", "Borland", "Breezecom", "Brocade", "Brother", "Brother Industries Ltd.", "BT", "Buffalo/MELCO", "Busybox", "Cabletron", "Capricorn Infotech India", "CareStream Health", "Carsten Schmitz", "Cayman", "Cellit", "Ceragon Networks", "CGI World", "Chase Research", "Check Point", "Chuming Chen", "Ciphertrust", "Cisco", "Cisco-Arrowpoint", "Citrix Systems, Inc.", "Claris", "ClearOne Communications", "Cobalt", "Colubris Networks", "Comcast SMC", "Comersus", "Comodo Group, Inc", "Compaq", "Compualynx", "Conceptronic", "Concord", "Conexant", "Control4", "CoronaMatrix", "Covertix", "Crystalview", "CTX International", "Cyberguard", "CyberMax", "Cyberoam", "Cyclades", "Daewoo", "Dallas Semiconductors", "Dassault Systemes", "Data General", "Datacom", "DataWizard Technologies Inc.", "Davolink", "Daytek", "Debian", "Deerfield", "Dell", "Develcon", "Dictaphone", "Digicorp", "Digicraft Software", "Digital Equipment", "Digium, Inc.", "Divar", "Dlink", "D-Link", "DotNetNuke Corporation", "Draytek", "Draytek Corp", "Ducati Motor Holding", "DuPont", "DVB", "Dynalink", "Dynix Library Systems", "Eaton", "Echelon Corporation", "Edimax", "Efficient", "Ektron, Inc.", "Elron", "Elsa", "EMC", "Eminent", "Enhydra", "Enox", "Enterasys", "Entrust", "EPiServer AB", "Epox", "eQ-3", "Ericsson", "eSeSIX Computer GmbH", "Exabyte", "Exacq Technologies", "Exinda Networks", "Extended Systems", "Extreme Networks", "eZ Systems", "EZPhotoSales", "F5", "Fastream Technologies", "Fastwire", "FatWire", "Firebird Project", "Flowpoint", "Fortinet", "Foscam", "Foundry Networks", "Freetech", "FrontRange Solutions", "Funk Software", "Galacticomm", "Gandalf", "Gateway", "GE Security, Inc.", "Geeklog", "General Instruments", "glFtpD", "Globespan Virata", "Google", "Gossamer Threads Inc.", "GrandStream", "Grandstream Networks, Inc", "Groupee, Inc.", "GuardOne", "H2O Project", "Hayes", "Hemoco Software", "Hewlett-Packard", "HighPOint", "Honeynet Project", "Horizon DataSys", "Hosting Controller", "HP", "Huawei Technologies Co", "Hyperic, Inc.", "IBM", "Inedo", "Informix", "Infrant", "Innovaphone", "Inova", "Integrated Networks", "Intel", "IntelliTouch", "Interbase", "iNTERFACEWARE Inc.", "Intermec", "Internet Archive", "Intersystems", "Inventel Wanadoo", "Ipswitch", "IQinVision", "IRC", "IronPort", "Iwill", "JAMF Software", "Janitza", "Jaspersoft Corporation", "JD Edwards", "Jean-Philippe Lang", "Jeebles Technology", "Jetform", "JetWay", "Johnson Controls", "Joss Technology", "Juniper", "Justin Hagstrom", "Kentico Software", "Kethinov", "Kodak", "Konica Minolta", "Kronos", "Kyocera", "LaCie", "LANSA", "Lantronics", "Lantronix", "Leading Edge", "Lenel", "Level1", "LGIC", "Liebert", "Lindsay Electronics", "Linksys", "Linux", "Liquidware Labs, Inc.", "Livingston", "Loglogic", "Longshine", "Lucent", "Luxon Communications", "M Technology", "MachSpeed", "Macsense", "Magento", "Magic-Pro", "Mambo", "ManageEngine", "Mandarin Library Automation", "Mantis", "Megastar", "Mentec", "Mercury Interactive", "Michiel", "Microcom", "Micron", "Micronet", "Micronics", "Microplex", "Microsoft", "Mike Peters", "MikroTik", "Mintel", "Mitel Networks", "mklencke", "Mole", "Motive", "Motorola", "MP3Mystic", "Muze", "MyioSoft", "MySQL", "Nanoteq", "NCR", "NEC", "NetApp", "Netasq", "NetBotz", "Netcomm", "Netcordia", "Netgear", "NetGenesis", "Netopia", "Netscape", "Netscreen", "Netstar", "NetworkICE", "NewMedia-NET GmbH", "Nexsan", "NeXT", "NICE Systems Ltd.", "Niksun", "Nimble", "Nokia", "Nortel", "Novell", "NSI", "Nurit", "ODS", "Oleg Khabarov", "Openlink", "OpenMarket", "Open-Xchange Inc.", "Optivision", "Oracle", "Osicom", "OTRS Inc.", "Overland Storage", "OvisLink Canada Inc.", "Packard Bell", "Packeteer", "Panasonic", "Pandatel", "Parallels", "Parrot", "Patton", "PBX", "phpLiteAdmin", "phpMyAdmin", "PHPReactor", "phpTest", "Pikatel", "Pirelli", "Pivotal Software, Inc.", "PlainTree", "PLANET Technology Corp.", "PokerTracker Software", "Pollsafe", "Polycom", "PostgreSQL", "Powerchute", "POWERLOGIC", "Prestige", "Prime", "PrimeBase", "Promise Technology, Inc.", "Proxicast", "Proxim", "PRTG", "QDI", "Q-Tec", "QualiTeam", "Quantex", "Quest Software", "Questra Corporation", "Radvision", "Raidzone", "Rainbow", "Rainer Wichmann", "Ramp Networks", "RapidStream", "Raritan", "Raritan Inc.", "RayTalk", "RCA", "Red Hat, Inc", "Redcreek Communications", "Remedy", "remote-exploit", "Research", "Research Machines", "Resumix", "Ricoh", "Rizen", "RM", "RNN", "Rodopi", "rPath", "RSA", "Saba", "Safecom", "Samba", "Sambar Technologies", "Samsung", "Samuel Abels", "SAP", "Schneider Electric", "Seagull Scientific", "Seclore", "Securicor3NET", "Semaphore", "Seyeon Technology Co., Ltd", "Shiva", "Shuttle", "Siemens", "Siemens Corp", "Sierra Wireless", "Silex Technology", "Silicon Graphics", "Site Interactive", "Sitecom", "Sitecore Corporation", "Skybox Security", "SMA America", "SMC", "Smith & Bentzen", "Snap Appliance", "Snom", "Software AG", "SolarWinds", "Sonicwall", "Sony", "Sony Ericsson", "Sourcefire", "Sovereign Hill", "Spider Systems", "Splunk", "Ssangyoung", "Stan Ozier", "Sun", "Super Micro Computer, Inc.", "SuperMicro", "Surecom", "Sweex", "Sybase", "Symantec", "Symbol Technologies, Inc", "Synology Inc", "SysKonnect", "Tandberg", "Tasman", "Tegile", "Telappliant", "Telebit", "Telelec", "Teletronics", "Telewell", "Telus", "Terayon", "test", "TexBox", "TextPortal", "Thecus Tech", "TIBCO", "Tim Schaab", "TimeTools", "Tiny", "Tinys", "TMC", "TopLayer", "Toshiba", "TOTOLINK", "Trend Micro", "Trintech", "Triumph-Adler", "Tumbleweed", "TYPO3", "Typo3 Association", "U.S. Robotics", "Unex", "Unidesk", "Unify", "United Technologies Corporation", "UNIX", "Unknown", "USRobotics", "UT Lexar", "UTStarcom", "VBrick Systems", "Verifone", "Veritas", "Verity", "Verizon", "vertex", "Vextrec Technology", "VieNuke", "Vina Technologies", "Virtual Programming", "Visa VAP", "Vobis", "VoiceGenie Technologies", "VoiceObjects Germany", "Vonage", "VPASP", "Wanco, Inc.", "WatchGuard", "Web Wiz", "Webmin", "Webramp", "Westell", "Wim Bervoets", "Wireless, Inc.", "WWWBoard", "Xavi", "Xerox", "Xinit Systems Ltd.", "X-Micro", "Xylan", "Xyplex", "Yakumo", "Yuxin", "Zcomax", "Zebra Technologies", "Zenith", "ZEOS", "Zeus", "Zoom", "Zyxel" ]
		self.website = "https://cirt.net/passwords?criteria={search}"

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_router'), debug_module=True)

	def getDefaultLogin(self, routerBrand='cisco'):
		formatedBrand = urllib.parse.quote(routerBrand)
		url = self.website.format(search=formatedBrand)
		response = requests.get( url )
		soup = BeautifulSoup(response.content)
		finalResponse = []

		for links in soup.find_all('table'):
			rows = links.find_all('tr')

			user = rows[1].find_all('td')[1].text
			if user == '(none)' or user == 'n.a':
				user = ''

			password = rows[2].find_all('td')[1].text
			if password == '(none)' or password == 'n.a':
				password = ''

			if not (user, password) in finalResponse:
				finalResponse.append( (user, password) )

		return finalResponse

	def getBrands(self):
		return self.brands


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os
import csv
import json
import xlsxwriter
import xmltodict
import xml.etree.ElementTree as ET
import pandas as pd
from pandas.io.json import json_normalize

config = Config.getConfig(parentKey='modules', key='ht_parser')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = ''

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_parser'), debug_module=True)

	def readFileToType(self, filename, typeToExport, typeOf=None, csv_headers=False):
		# Recogemos la extensión
		extension_file = filename.split('.')[-1]

		# Validamos parametros de filename y typeOf
		is_parseable = False
		if not extension_file.lower() in ('csv', 'html', 'json', 'xml'):
			if typeOf and typeOf.lower() in ('csv', 'html', 'json', 'xml'):
				is_parseable = True
		else:
			is_parseable = True

		# Si podemos parsearlo
		if is_parseable:
			# Transformar CSV a X
			if (extension_file or typeOf) == 'csv' and typeToExport == 'json':
				datos = self.__readCSVFile__(filename)
				return self.__csvToJSON__( datos, csv_headers )

			if (extension_file or typeOf) == 'csv' and typeToExport == 'xml':
				self.__csvToXML__()

			if (extension_file or typeOf) == 'csv' and typeToExport == 'html':
				self.__csvToHTML__()

			# Transformar JSON a X
			if (extension_file or typeOf) == 'json' and typeToExport == 'csv':
				datos = self.__readJSONFile__(filename)
				return self.__jsonToCSV__( datos )

			if (extension_file or typeOf) == 'json' and typeToExport == 'xml':
				self.__jsonToXML__()

			if (extension_file or typeOf) == 'json' and typeToExport == 'html':
				self.__jsonToHTML__()

			# Transformar XML a X
			if (extension_file or typeOf) == 'xml' and typeToExport == 'json':
				datos = self.__readXMLFile__(filename)
				return self.__xmlToJSON__( datos )

			if (extension_file or typeOf) == 'xml' and typeToExport == 'csv':
				self.__xmlToCSV__()

			if (extension_file or typeOf) == 'xml' and typeToExport == 'html':
				self.__xmlToHTML__()

			# Transformar HTML a X
			if (extension_file or typeOf) == 'html' and typeToExport == 'csv':
				self.__htmlToCSV__()

			if (extension_file or typeOf) == 'html' and typeToExport == 'xml':
				self.__htmlToXML__()

			if (extension_file or typeOf) == 'html' and typeToExport == 'json':
				self.__htmlToJSON__()

		else:
			return None
		
	def __readCSVFile__(self, filename):
		try:
			csvfile = open(filename, 'r')
			reader = csv.reader(csvfile)
			return reader
		except:
			return None

	def __readXMLFile__(self, filename):
		try:
			xmlfile = open(filename, 'r')
			datos = ET.parse(xmlfile).getroot()
			return datos
		except Exception as e:
			return str( e )

	def __readJSONFile__(self, filename):
		try:
			jsonfile = open(filename, 'r')
			datos = json.load( jsonfile )
			return datos
		except Exception as e:
			return str( e )

	def __csvToJSON__(self, fileData, csv_headers=False):
		data = {}

		headers = None

		temp_data = []

		for row in fileData:
			temp_data.append(row)

		if csv_headers:
			headers = temp_data[0]
			del temp_data[0]

		for i_row, row in enumerate(temp_data):
			
			if headers:
				data[ str( i_row ) ] = {}

				for i_col, col in enumerate(row):
					
					data[ str( i_row ) ][ headers[ i_col ] ] = col
			else:
				data[ str( i_row ) ] = row

		return data

	def __csvToXML__(self):
		pass
	
	def __csvToHTML__(self):
		pass

	def __jsonToCSV__(self, fileData):
		return json_normalize(fileData).to_csv( encoding='utf-8' )

	def __jsonToXML__(self):
		pass
	
	def __jsonToHTML__(self):
		pass

	def __xmlToCSV__(self):
		pass

	def __xmlToJSON__(self, fileData):
		try:
			parsedString = ET.tostring( fileData, encoding='unicode' )
			return json.dumps( xmltodict.parse(parsedString) )
		except Exception as e:
			return str( e )
	
	def __xmlToHTML__(self):
		pass

	def __htmlToCSV__(self):
		pass

	def __htmlToJSON__(self):
		pass
	
	def __htmlToXML__(self):
		pass



from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os
from pysia import Sia

config = Config.getConfig(parentKey='modules', key='ht_sia')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():
	
	def __init__(self):
		self._main_gui_func_ = ''
		self.sc = None

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_sia'), debug_module=True)

	def getDaemonConstants(self):
		self.__startSIA__()
		return self.sc.get_daemon_constants()

	def getDaemonVersion(self):
		self.__startSIA__()
		return self.sc.get_daemon_version()

	def getConsensus(self):
		self.__startSIA__()
		return self.sc.get_consensus()

	def getRenter(self):
		self.__startSIA__()
		return self.sc.get_renter()

	def getRenterContracts(self):
		self.__startSIA__()
		return self.sc.get_renter_contracts()

	def getRenterDownloads(self):
		self.__startSIA__()
		return self.sc.get_renter_downloads()

	def getRenterFiles(self):
		self.__startSIA__()
		return self.sc.get_renter_files()

	def getRenterPrices(self):
		self.__startSIA__()
		return self.sc.get_renter_prices()

	def getWalletTransactions(self):
		self.__startSIA__()
		return self.sc.get_wallet_transactions()

	def getWalletSeeds(self):
		self.__startSIA__()
		return self.sc.get_wallet_seeds()

	def getWalletBackup(self):
		self.__startSIA__()
		return self.sc.get_wallet_backup()

	def getWallet(self):
		self.__startSIA__()
		return self.sc.get_wallet()

	def getWalletAddress(self):
		self.__startSIA__()
		return self.sc.get_wallet_address()

	def getWalletAddresses(self):
		self.__startSIA__()
		return self.sc.get_wallet_addresses()

	def __startSIA__(self):
		if not self.sc:
			try:
				self.sc = Sia()
			except:
				return None
		return self.sc

	def __stopSIA__(self):
		if self.sc:
			return self.sc.get_daemon_stop()
		return None



from . import Logger, Utils
import json
import time
from datetime import datetime
import requests
import os as __os
import textwrap as __textwrap
import urllib.parse
import threading as __threading
from abc import ABC as __ABC
from abc import abstractmethod
from functools import wraps
import errno
from collections import OrderedDict
import binascii
from Crypto.Hash import SHA
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5

import hackingtools as ht

class Host():

    def __init__(self, target_id, ip=[], ports=[], vulnerabilities=[], hostnames=[], city='', country_name='', org='', isp='', last_update='', asn='', os='', data=''):
        self.target_id = target_id
        self.ip = ip
        self.ports = ports
        self.vulnerabilities = vulnerabilities
        self.hostnames = hostnames
        self.city = city
        self.country_name = country_name
        self.org = org
        self.isp = isp
        self.last_update = last_update
        self.asn = asn
        self.os = os
        self.data = data

    def __str__(self):
        parameters = {}
        if self.target_id:
            parameters["target_id"] = self.getTargetId()
        if self.ip:
            parameters["ip"] = self.getIp()
        if self.ports:
            parameters["ports"] = self.getPorts()
        if self.vulnerabilities:
            parameters["vulnerabilities"] = self.getVulnerabilities()
        if self.hostnames:
            parameters["hostnames"] = self.getHostNames()
        if self.city:
            parameters["city"] = self.getCity()
        if self.country_name:
            parameters["country_name"] = self.getCountryName()
        if self.org:
            parameters["org"] = self.getOrganitation()
        if self.isp:
            parameters["isp"] = self.getISP()
        if self.last_update:
            parameters["last_update"] = self.getLastUpdate()
        if self.asn:
            parameters["asn"] = self.getASN()
        if self.os:
            parameters["os"] = self.getOS()
        if self.data:
            parameters["data"] = self.getData()
        Logger.printMessage(message='{data}'.format(data=json.dumps(parameters, indent=4, sort_keys=True), debug_core=True))
        return parameters
    
    def setTarget(self, id):
        self.target_id = id

    def addIp(self, ip):
        self.removeIp(ip)
        self.ip.append(ip)

    def removeIp(self, ip):
        self.ip.remove(ip)

    def addPort(self, port):
        self.removePort(port)
        self.ports.append(port)

    def removePort(self, port):
        self.ports.remove(port)

    def addVulnerability(self, vuln):
        if not vuln in self.vulnerabilities:
            self.vulnerabilities.append(vuln)

    def removeVulnerability(self, vuln):
        if vuln in self.vulnerabilities:
            self.vulnerabilities.remove(vuln)

    def addHostName(self, hostname):
        self.removeHostName(hostname)
        self.hostnames.append(hostname)

    def removeHostName(self, hostname):
        self.hostnames.remove(hostname)

    def setCity(self, city):
        self.city = city

    def setCountryname(self, countryname):
        self.country_name = countryname

    def setOrganitation(self, organitation):
        self.org = organitation

    def setISP(self, isp):
        self.isp = isp

    def setLastUpdate(self, lastupdate):
        self.last_update = lastupdate

    def setASN(self, asn):
        self.asn = asn

    def setOS(self, os):
        self.os = os

    def setData(self, data):
        self.data = data

    def getTargetId(self):
        return self.target_id

    def getIp(self):
        return self.ip

    def getPorts(self):
        return self.ports

    def getVulnerabilities(self):
        return ', '.join([vuln.getCVE() for vuln in self.vulnerabilities])

    def getHostNames(self):
        return self.hostnames

    def getCity(self):
        return self.city

    def getCountryName(self):
        return self.country_name

    def getOrganitation(self):
        return self.org

    def getISP(self):
        return self.isp

    def getLastUpdate(self):
        return self.last_update

    def getASN(self):
        return self.asn

    def getOS(self):
        return self.os

    def getData(self):
        return self.data

    def addScanResult(self, result):
        try:
            Logger.printMessage(message='addScanResult', description=result, debug_core=True)
            for r in result:
                try:
                    if "ip_str" == r:
                        self.addIp(result[r])
                    if "ports" == r:
                        for p, val in result[r]:
                            self.addPort({p, val})
                    if "hostnames" == r:
                        self.addHostName(list(result[r]))
                    if "city" == r:
                        self.addCity(result[r])
                    if "country_name" == r:
                        self.addCountryName(result[r])
                    if "org" == r:
                        self.addOrganitation(result[r])
                    if "isp" == r:
                        self.addISP(result[r])
                    if "last_update" == r:
                        self.addLastUpdate(result[r])
                    if "asn" == r:
                        self.addASN(result[r])
                    if "os" == r:
                        self.addOS(result[r])
                    if "data" == r:
                        self.addData(result[r])
                except:
                    pass
        except:
            pass

class Target():

    def __init__(self, name, id, mail=[], passwords=[], hosts=[], pictures=[], social_networks=[]):
        self.name = name
        self.id = id
        self.mail = mail
        self.passwords = passwords
        self.hosts = hosts
        self.pictures = pictures
        self.social_networks = social_networks

    def renameTarget(self, name):
        try:
            self.name = name
            return True
        except:
            return False

    def addHost(self, host):
        try:
            if not host in self.hosts:
                self.hosts.append(host)
                return True
            return False
        except:
            return False

    def removeHost(self, host):
        try:
            if host in self.hosts:
                self.hosts.remove(host)
                return True
            return False
        except:
            return False

    def getHostsWithKey(self, key):
        try:
            response = []
            for host in self.hosts:
                if key in host.name:
                    response.append(host)
            return response
        except:
            return []

    def getHostByName(self, name):
        try:
            response = []
            for host in self.hosts:
                if name in host.name:
                    response.append(host)
            return response
        except:
            return []

    def getHostByCpe(self, cpe):
        try:
            response = []
            for host in self.hosts:
                if name in host.name:
                    response.append(host)
            return response
        except:
            return []

    def getHostsByIp(self, ip):
        try:
            response = []
            for host in self.hosts:
                if ip in host.ip:
                    response.append(host)
            return response
        except:
            return None

    def existsHostWithIp(self, ip):
        try:
            for host in self.hosts:
                if ip in host.ip:
                    return True
            return False
        except:
            return None
    
class Scan():
    
    def __init__(self, name, modules=[], hosts=[]):
        self.name = name
        self.modules = list(modules)
        self.hosts = hosts

    def rename(self, newName):
        self.name = newName

    def addModule(self, module):
        self.removeModule(module) # If exists
        self.modules.append(module)

    def removeModule(self, module):
        self.modules.remove(module)

    def addHost(self, host):
        self.removeHost(host)
        self.hosts.append(host)

    def removeHost(self, host_id):
        self.hosts.remove(host_id)

    def getName(self):
        return self.name

    def getModules(self):
        return self.modules

    def getHosts(self):
        return self.hosts

class Vulnerability():
    
    def __init__(self, name, cve='CVE-0000-0000', cpe='', cwe='', cvss_base=0):
        self.name = name
        self.cve = cve
        self.cwe = cwe
        self.cvss_base = cvss_base

    def getName(self):
        return self.name

    def setName(self, name):
        self.name = name

    def getCVE(self):
        return self.cve

    def setCVE(self, cve):
        self.cve = cve

    def getCWE(self):
        return self.cwe

    def setCWE(self, cwe):
        self.cwe = cwe

    def getCPE(self):
        return self.cpe

    def setCPE(self, cpe):
        self.cpe = cpe

    def getCVSS(self):
        return self.cvss_base

    def setCVSS(self, cvss):
        self.cvss_base = cvss

class Historical():
    
    def __init__(self):
        self.data = {}

    def addData(self, data):
        self.data[Utils.getTime()] = data

    def getData(self):
        return self.data
        
class Worker():
    
    def __init__(self): 
        self._running = True
        self.responses = []
        self.time = None
        self.timeout = None

    def is_alive(self):
        return self._running

    def terminate(self): 
        self._running = False
        raise KeyboardInterrupt
    
    def getLastResponse(self):
        return self.responses[-1]

    def run(self, functionCall, args, timesleep, loop, timeout):
        if timeout and isinstance(timeout, int) and not self.time:
            self.time = datetime.now()
            self.timeout = int(timeout)

        if loop:

            while self._running:
                try:
                    if timeout and (datetime.now() - self.time).total_seconds() > self.timeout:
                        self.terminate()

                    else:
                        func = '{f}{a}'.format(f=functionCall, a=tuple(args, ))
                        res = eval(func)
                        self.responses.append( res )
                        time.sleep(timesleep)
                except KeyboardInterrupt:
                    pass

        else:
            try:
                if timeout and (datetime.now() - self.time).total_seconds() > self.timeout:
                    self.terminate()

                else:
                    func = '{f}{a}'.format(f=functionCall, a=tuple(args, ))
                    res = eval(func)
                    self.responses.append( res )
                    self.terminate()
            except KeyboardInterrupt:
                pass

# Instagram Module Main Utils

class RequestHandler:
	"""
	Handle all requests with specific user-agent
	"""
	def __init__(self, user_agent, ret="text"):
		self.user_agent = user_agent
		self.current_proxy = None
		self.ret = ret

	@property
	def proxy(self):
		return self.proxy

	@proxy.setter
	def proxy(self, new_proxy):
		self.current_proxy = new_proxy

	def get(self, url):
		"""
		make request
		:param url:
		:return:
		"""
		proxies = {
			"http": f"http://{self.current_proxy}",
			"https": f"https://{self.current_proxy}"
		}
		headers = {
			"User-Agent": self.header_maker(self.user_agent)
		}
		try:
			s = requests.Session()
			if self.current_proxy:
				res = s.get(url, headers=headers, proxies=proxies)
			else:
				res = s.get(url, headers=headers)
			if res.status_code == 200:
				# check return mode
				if self.ret == "text":
					return res.text
				else:
					return res.json()
			else:
				return None
		except requests.exceptions.ConnectionError:
			raise requests.exceptions.ConnectionError
		except json.decoder.JSONDecodeError:
			return None

	def header_maker(self, mode):
		"""
		make header and return as dict
		:param mode:
		:return:
		"""
		user_agents = {
			"FF": "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Safari/537.36",
			"TIMELINE": "Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/40.0.2214.93 Safari/537.36",
		}

		return user_agents[mode]

class CookieSessionManager:

    def __init__(self, session_folder, filename):
        self.session_folder = session_folder
        self.filename = filename

    def get_saved_cookies(self):
        try:
            f = open(self.session_folder + self.filename, 'r') 
            return f.read()
        except FileNotFoundError:
            return None

    def set_saved_cookies(self, cookie_string):
        if not __os.path.exists(self.session_folder):
            __os.makedirs(self.session_folder)

        with open(self.session_folder + self.filename,"w+") as f:
            f.write(cookie_string)

    def empty_saved_cookies(self):
        try:
            __os.remove(self.session_folder + self.filename)
        except FileNotFoundError:
            pass

class InitializerModel:

    def __init__(self, props=None):

        self._is_new = True
        self._is_loaded = False
        """init data was empty"""
        self._is_load_empty = True
        self._is_fake = False
        self._modified = None

        """Array of initialization data"""
        self._data = {}

        self.modified = time.time()

        if props is not None and len(props) > 0:
            self._init(props)

    def _init(self, props):
        """

        :param props: props array
        :return: None
        """
        for key in props.keys():
            try:
                self._init_properties_custom(props[key], key, props)
            except AttributeError:
                # if function does not exist fill help data array
                self._data[key] = props[key]

        self._is_new = False
        self._is_loaded = True
        self._is_load_empty = False

class Account(InitializerModel):

    def __init__(self, props=None):
        self.identifier = None
        self.username = None
        self.full_name = None
        self.profile_pic_url = None
        self.profile_pic_url_hd = None
        self.biography = None
        self.external_url = None
        self.follows_count = 0
        self.followed_by_count = 0
        self.media_count = 0
        self.is_private = False
        self.is_verified = False
        self.medias = []
        self.blocked_by_viewer = False
        self.country_block = False
        self.followed_by_viewer = False
        self.follows_viewer = False
        self.has_channel = False
        self.has_blocked_viewer = False
        self.highlight_reel_count = 0
        self.has_requested_viewer = False
        self.is_business_account = False
        self.is_joined_recently = False
        self.business_category_name = None
        self.business_email = None
        self.business_phone_number = None
        self.business_address_json = None
        self.requested_by_viewer = False
        self.connected_fb_page = None

        super(Account, self).__init__(props)

    def get_profile_picture_url(self):
        try:
            if not self.profile_pic_url_hd == '':
                return self.profile_pic_url_hd
        except AttributeError:
            try:
                return self.profile_pic_url
            except AttributeError:
                return ''

    def get_data(self):
        data = {}
        values = ('identifier', 'username', 'full_name', 'biography', 'external_url', 'media_count', 'followed_by_count', 'follows_count', 'is_private', 'is_verified')
        for val in values:
            if hasattr(self, val):
                data[val] = eval('self.{val}'.format(val=val))
        data['picture'] = self.get_profile_picture_url()
        return data

    def __str__(self):
        string = f"""
        Account info:
        Id: {self.identifier}
        Username: {self.username if hasattr(self, 'username') else '-'}
        Full Name: {self.full_name if hasattr(self, 'full_name') else '-'}
        Bio: {self.biography if hasattr(self, 'biography') else '-'}
        Profile Pic Url: {self.get_profile_picture_url()}
        External url: {self.external_url if hasattr(self, 'external_url') else '-'}
        Number of published posts: {self.media_count if hasattr(self, 'media_count') else '-'}
        Number of followers: {self.followed_by_count if hasattr(self, 'followed_by_count') else '-'}
        Number of follows: {self.follows_count if hasattr(self, 'follows_count') else '-'}
        Is private: {self.is_private if hasattr(self, 'is_private') else '-'}
        Is verified: {self.is_verified if hasattr(self, 'is_verified') else '-'}
        """
        return __textwrap.dedent(string)

    """
     * @param Media $media
     * @return Account
    """
    def add_media(self, media):
        try:
            self.medias.append(media)
        except AttributeError:
            raise AttributeError

    def _init_properties_custom(self, value, prop, array):
        
        if prop == 'id':
            self.identifier = value

        standart_properties = [
            'username',
            'full_name',
            'profile_pic_url',
            'profile_pic_url_hd',
            'biography',
            'external_url',
            'is_private',
            'is_verified',
            'blocked_by_viewer',
            'country_block',
            'followed_by_viewer',
            'follows_viewer',
            'has_channel',
            'has_blocked_viewer', 
            'highlight_reel_count',
            'has_requested_viewer',
            'is_business_account',
            'is_joined_recently',
            'business_category_name',
            'business_email',
            'business_phone_number',
            'business_address_json',
            'requested_by_viewer',
            'connected_fb_page'
        ]
        if prop in standart_properties:
            self.__setattr__(prop, value)   
        
        if prop == 'edge_follow':
            self.follows_count = array[prop]['count'] \
                if array[prop]['count'] is not None  else 0

        if prop == 'edge_followed_by':
            self.followed_by_count = array[prop]['count'] \
                if array[prop]['count'] is not None else 0

        if prop == 'edge_owner_to_timeline_media':
            self._init_media(array[prop])

    def _init_media(self, array):
        self.media_count = array['count'] if 'count' in array.keys() else 0 

        try:
            nodes = array['edges']
        except:
            return

        if not self.media_count or isinstance(nodes, list):
            return

        for media_array in nodes:
            media = Media(media_array['node'])
            if isinstance(media, Media):
                self.add_media(media)

class Comment(InitializerModel):
    """
     * @param $value
     * @param $prop
     """

    def __init__(self, props=None):
        self.identifier = None
        self.text = None
        self.created_at = None
        # Account object
        self.owner = None

        super(Comment, self).__init__(props)

    def _init_properties_custom(self, value, prop, array):

        if prop == 'id':
           self.identifier = value

        standart_properties = [
            'created_at',
            'text',
        ]

        if prop in standart_properties:
            self.__setattr__(prop, value)

        if prop == 'owner':
            self.owner = Account(value)

class Media(InitializerModel):
    TYPE_IMAGE = 'image'
    TYPE_VIDEO = 'video'
    TYPE_SIDECAR = 'sidecar'
    TYPE_CAROUSEL = 'carousel'

    def __init__(self, props=None):
        self.identifier = None
        self.short_code = None
        self.created_time = 0
        self.type = None
        self.link = None
        self.image_low_resolution_url = None
        self.image_thumbnail_url = None
        self.image_standard_resolution_url = None
        self.image_high_resolution_url = None
        self.square_images = []
        self.carousel_media = []
        self.caption = None
        self.is_ad = False
        self.video_low_resolution_url = None
        self.video_standard_resolution_url = None
        self.video_low_bandwidth_url = None
        self.video_views = 0
        self.video_url = None
        # account object
        self.owner = None
        self.likes_count = 0
        self.location_id = None
        self.location_name = None
        self.comments_count = 0
        self.comments = []
        self.has_more_comments = False
        self.comments_next_page = None
        self.location_slug = None

        super(Media, self).__init__(props)

    @staticmethod
    def get_id_from_code(code):
        alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_'
        id = 0

        for i in range(len(code)):
            c = code[i]
            id = id * 64 + alphabet.index(c)

        return id

    @staticmethod
    def get_link_from_id(id):
        code = Media.get_code_from_id(id)
        return InstagramEndPoints().get_media_page_link(code)

    @staticmethod
    def get_code_from_id(id):
        parts = str(id).partition('_')
        id = int(parts[0])
        alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_'
        code = ''

        while (id > 0):
            remainder = int(id) % 64
            id = (id - remainder) // 64
            code = alphabet[remainder] + code

        return code

    def __str__(self):
        string = f"""
        Media Info:
        'Id: {self.identifier}
        Shortcode: {self.short_code}
        Created at: {self.created_time}
        Caption: {self.caption}
        Number of comments: {self.comments_count if hasattr(self,
                                                            'commentsCount') else 0}
        Number of likes: {self.likes_count}
        Link: {self.link}
        Hig res image: {self.image_high_resolution_url}
        Media type: {self.type}
        """

        return __textwrap.dedent(string)

    def _init_properties_custom(self, value, prop, arr):

        if prop == 'id':
            self.identifier = value

        standart_properties = [
            'type',
            'link',
            'thumbnail_src',
            'caption',
            'video_view_count',
            'caption_is_edited',
            'is_ad'
        ]

        if prop in standart_properties:
            self.__setattr__(prop, value)

        elif prop == 'created_time' or prop == 'taken_at_timestamp' or prop == 'date':
            self.created_time = int(value)

        elif prop == 'code':
            self.short_code = value
            self.link = InstagramEndPoints().get_media_page_link(self.short_code)

        elif prop == 'comments':
            self.comments_count = arr[prop]['count']
        elif prop == 'likes':
            self.likes_count = arr[prop]['count']

        elif prop == 'display_resources':
            medias_url = []
            for media in value:
                medias_url.append(media['src'])

                if media['config_width'] == 640:
                    self.image_thumbnail_url = media['src']
                elif media['config_width'] == 750:
                    self.image_low_resolution_url = media['src']
                elif media['config_width'] == 1080:
                    self.image_standard_resolution_url = media['src']

        elif prop == 'display_src' or prop == 'display_url':
            self.image_high_resolution_url = value
            if self.type is None:
                self.type = Media.TYPE_IMAGE

        elif prop == 'thumbnail_resources':
            square_images_url = []
            for square_image in value:
                square_images_url.append(square_image['src'])
            self.square_images = square_images_url

        elif prop == 'carousel_media':
            self.type = Media.TYPE_CAROUSEL
            self.carousel_media = []
            for carousel_array in arr["carousel_media"]:
                self.set_carousel_media(arr, carousel_array)

        elif prop == 'video_views':
            self.video_views = value
            self.type = Media.TYPE_VIDEO

        elif prop == 'videos':
            self.video_low_resolution_url = arr[prop]['low_resolution']['url']
            self.video_standard_resolution_url = \
            arr[prop]['standard_resolution']['url']
            self.video_low_bandwith_url = arr[prop]['low_bandwidth']['url']

        elif prop == 'video_resources':
            for video in value:
                if video['profile'] == 'MAIN':
                    self.video_standard_resolution_url = video['src']
                elif video['profile'] == 'BASELINE':
                    self.video_low_resolution_url = video['src']
                    self.video_low_bandwith_url = video['src']

        elif prop == 'location' and value is not None:
            self.location_id = arr[prop]['id']
            self.location_name = arr[prop]['name']
            self.location_slug = arr[prop]['slug']

        elif prop == 'user' or prop == 'owner':
            self.owner = Account(arr[prop])

        elif prop == 'is_video':
            if bool(value):
                self.type = Media.TYPE_VIDEO

        elif prop == 'video_url':
            self.video_standard_resolution_url = value

        elif prop == 'shortcode':
            self.short_code = value
            self.link = InstagramEndPoints().get_media_page_link(self.short_code)

        elif prop == 'edge_media_to_comment':
            try:
                self.comments_count = int(arr[prop]['count'])
            except KeyError:
                pass
            try:
                edges = arr[prop]['edges']

                for comment_data in edges:
                    self.comments.append(Comment(comment_data['node']))
            except KeyError:
                pass
            try:
                self.has_more_comments = bool(
                    arr[prop]['page_info']['has_next_page'])
            except KeyError:
                pass
            try:
                self.comments_next_page = str(
                    arr[prop]['page_info']['end_cursor'])
            except KeyError:
                pass

        elif prop == 'edge_media_preview_like':
            self.likes_count = arr[prop]['count']
        elif prop == 'edge_liked_by':
            self.likes_count = arr[prop]['count']

        elif prop == 'edge_media_to_caption':
            try:
                self.caption = arr[prop]['edges'][0]['node']['text']
            except (KeyError, IndexError):
                pass

        elif prop == 'edge_sidecar_to_children':
            pass
            # #TODO implement
            # if (!is_array($arr[$prop]['edges'])) {
            #     break;
            # }
            # foreach ($arr[$prop]['edges'] as $edge) {
            #     if (!isset($edge['node'])) {
            #         continue;
            #     }

            #     $this->sidecarMedias[] = static::create($edge['node']);
            # }
        elif prop == '__typename':
            if value == 'GraphImage':
                self.type = Media.TYPE_IMAGE
            elif value == 'GraphVideo':
                self.type = Media.TYPE_VIDEO
            elif value == 'GraphSidecar':
                self.type = Media.TYPE_SIDECAR

        # if self.ownerId and self.owner != None:
        #     self.ownerId = self.getOwner().getId()

    @staticmethod
    def set_carousel_media(media_array, carousel_array):

        print(carousel_array)
        # TODO implement
        pass
        """
        param mediaArray
        param carouselArray
        param instance
        return mixed
        """
        # carousel_media = CarouselMedia()
        # carousel_media.type(carousel_array['type'])

        # try:
        #     images = carousel_array['images']
        # except KeyError:
        #     pass

        # carousel_images = Media.__get_image_urls(
        #     carousel_array['images']['standard_resolution']['url'])
        # carousel_media.imageLowResolutionUrl = carousel_images['low']
        # carousel_media.imageThumbnailUrl = carousel_images['thumbnail']
        # carousel_media.imageStandardResolutionUrl = carousel_images['standard']
        # carousel_media.imageHighResolutionUrl = carousel_images['high']

        # if carousel_media.type == Media.TYPE_VIDEO:
        #     try:
        #         carousel_media.video_views = carousel_array['video_views']
        #     except KeyError:
        #         pass

        #     if 'videos' in carousel_array.keys():
        #         carousel_media.videoLowResolutionUrl(
        #             carousel_array['videos']['low_resolution']['url'])
        #         carousel_media.videoStandardResolutionUrl(
        #             carousel_array['videos']['standard_resolution']['url'])
        #         carousel_media.videoLowBandwidthUrl(
        #             carousel_array['videos']['low_bandwidth']['url'])

        # media_array.append(carousel_media)
        # # array_push($instance->carouselMedia, $carouselMedia);
        # return media_array

    @staticmethod
    def __getImageUrls(image_url):
        parts = '/'.split(urllib.parse.quote_plus(image_url)['path'])
        imageName = parts[len(parts) - 1]
        urls = {
            'thumbnail': InstagramEndPoints().INSTAGRAM_CDN_URL + 't/s150x150/' + imageName,
            'low': InstagramEndPoints().INSTAGRAM_CDN_URL + 't/s320x320/' + imageName,
            'standard': InstagramEndPoints().INSTAGRAM_CDN_URL + 't/s640x640/' + imageName,
            'high': InstagramEndPoints().INSTAGRAM_CDN_URL + 't/' + imageName,
        }
        return urls

class Location(InitializerModel):

    def __init__(self, props=None):
        self.identifier = None
        self.has_public_page = None
        self.name = None
        self.slug = None
        self.lat = None
        self.lng = None
        self.modified = None
        super(Location, self).__init__(props)

    def __str__(self):
        string = f"""
        Location info:
        Id: {self.identifier}
        Name: {self.name}
        Latitude: {self.lat}
        Longitude: {self.lng}
        Slug: {self.slug}
        Is public page available: {self.has_public_page}
        """

        return __textwrap.dedent(string)

    def _init_properties_custom(self, value, prop, arr):

        if prop == 'id':
            self.identifier = value

        standart_properties = [
            'has_public_page',
            'name',
            'slug',
            'lat',
            'lng',
            'modified',
        ]

        if prop in standart_properties:
            self.__setattr__(prop, value)

class Tag(InitializerModel):

    def __init__(self, props=None):
        self._media_count = 0
        self._name = None
        self._id = None
        super(Tag, self).__init__(props)

    def _init_properties_custom(self, value, prop, arr):

        if prop == 'id':
            self.identifier = value
        
        standart_properties = [
            'media_count',
            'name',
        ]

        if prop in standart_properties:
            self.__setattr__(prop, value)

class UserStories(InitializerModel):

    def __init__(self, stories=[], owner=None):
        if stories is None:
            stories = []
        self.owner = owner
        self.stories = stories
        super().__init__()

class Story(Media):

    skip_prop = [
        'owner'
    ]

    #  We do not need some values - do not parse it for Story,
    #  for example - we do not need owner object inside story
     
    #  param value
    #  param prop
    #  param arr

    def _init_properties_custom(self, value, prop, arr):
        if prop in Story.skip_prop:
            return
        
        super()._init_properties_custom(value, prop, arr)

    def __str__(self):
        string = f"""
        Story Info:
        'Id: {self.identifier}
        Hig res image: {self.image_high_resolution_url}
        Media type: {self.type if hasattr(self, 'type') else ''}
        """
        
        return __textwrap.dedent(string)

class CarouselMedia:

    def __init__(self):
        self.__type = ''
        self.__image_low_resolution_url = ''
        self.__image_thumbnail_url = ''
        self.__image_standard_resolution_url = ''
        self.__image_high_resolution_url = ''
        self.__video_low_resolution_url = ''
        self.__video_standard_resolution_url = ''
        self.__video_low_bandwidth_url = ''
        self.__video_views = ''

class InstagramEndPoints:
    
    USER_MEDIAS = '17880160963012870'
    USER_STORIES = '17890626976041463'
    STORIES = '17873473675158481'

    BASE_URL = 'https://www.instagram.com'
    LOGIN_URL = 'https://www.instagram.com/accounts/login/ajax/'
    ACCOUNT_PAGE = 'https://www.instagram.com/{s}'
    MEDIA_LINK = 'https://www.instagram.com/p/{s}'
    ACCOUNT_MEDIAS = 'https://www.instagram.com/graphql/query/?query_hash=42323d64886122307be10013ad2dcc44&variables={s}'
    ACCOUNT_JSON_INFO = 'https://www.instagram.com/{s}/?__a=1'
    MEDIA_JSON_INFO = 'https://www.instagram.com/p/{s}/?__a=1'
    MEDIA_JSON_BY_LOCATION_ID = 'https://www.instagram.com/explore/locations/{s}/?__a=1&max_id={s2}'
    MEDIA_JSON_BY_TAG = 'https://www.instagram.com/explore/tags/{s}/?__a=1&max_id={s2}'
    GENERAL_SEARCH = 'https://www.instagram.com/web/search/topsearch/?query={s}'
    COMMENTS_BEFORE_COMMENT_ID_BY_CODE = 'https://www.instagram.com/graphql/query/?query_hash=97b41c52301f77ce508f55e66d17620e&variables={s}'
    LIKES_BY_SHORTCODE_OLD = 'https://www.instagram.com/graphql/query/?query_id=17864450716183058&variables={"shortcode":"{s}","first":{s2},"after":"{s3}"}'
    LIKES_BY_SHORTCODE = 'https://www.instagram.com/graphql/query/?query_hash=d5d763b1e2acf209d62d22d184488e57&variables={s}'
    FOLLOWING_URL_OLD = 'https://www.instagram.com/graphql/query/?query_id=17874545323001329&id={{accountId}}&first={{count}}&after={{after}}'
    FOLLOWING_URL = 'https://www.instagram.com/graphql/query/?query_hash=d04b0a864b4b54837c0d870b0e77e076&variables={s}'
    FOLLOWERS_URL_OLD = 'https://www.instagram.com/graphql/query/?query_id=17851374694183129&id={{accountId}}&first={{count}}&after={{after}}'
    FOLLOWERS_URL = 'https://www.instagram.com/graphql/query/?query_hash=c76146de99bb02f6415203be841dd25a&variables={s}'
    FOLLOW_URL = 'https://www.instagram.com/web/friendships/{s}/follow/'
    UNFOLLOW_URL = 'https://www.instagram.com/web/friendships/{s}/unfollow/'
    INSTAGRAM_CDN_URL = 'https://scontent.cdninstagram.com/'
    ACCOUNT_JSON_PRIVATE_INFO_BY_ID = 'https://i.instagram.com/api/v1/users/{s}/info/'
    LIKE_URL = 'https://www.instagram.com/web/likes/{s}/like/'
    UNLIKE_URL = 'https://www.instagram.com/web/likes/{s}/unlike/'
    ADD_COMMENT_URL = 'https://www.instagram.com/web/comments/{s}/add/'
    DELETE_COMMENT_URL = 'https://www.instagram.com/web/comments/{s}/delete/{s2}/'

    ACCOUNT_MEDIAS2 = 'https://www.instagram.com/graphql/query/?query_id=17880160963012870&id={{accountId}}&first=10&after='

    GRAPH_QL_QUERY_URL = 'https://www.instagram.com/graphql/query/?query_id={s}'

    request_media_count = 30

    def get_account_page_link(self, username):
        return self.ACCOUNT_PAGE.format( s=urllib.parse.quote_plus(username) )

    def get_account_json_link(self, username):
        return self.ACCOUNT_JSON_INFO.format( s=urllib.parse.quote_plus(username) )

    def get_account_json_private_info_link_by_account_id(self, account_id):
        return self.ACCOUNT_JSON_PRIVATE_INFO_BY_ID.format( s=urllib.parse.quote_plus(str(account_id)) )

    def get_account_medias_json_link(self, variables):
        return self.ACCOUNT_MEDIAS.format( s=urllib.parse.quote_plus(json.dumps(variables, separators=(',', ':'))) )

    def get_media_page_link(self, code):
        return self.MEDIA_LINK.format( s=urllib.parse.quote_plus(code) )

    def get_media_json_link(self, code):
        return self.MEDIA_JSON_INFO.format( s=urllib.parse.quote_plus(code) )

    def get_medias_json_by_location_id_link(self, facebook_location_id, max_id=''):
        return self.MEDIA_JSON_BY_LOCATION_ID.format( s=urllib.parse.quote_plus(str(facebook_location_id)), s2=urllib.parse.quote_plus(max_id) )

    def get_medias_json_by_tag_link(self, tag, max_id=''):
        return self.MEDIA_JSON_BY_TAG.format( s=urllib.parse.quote_plus(str(tag)), s2=urllib.parse.quote_plus(str(max_id)) )

    def get_general_search_json_link(self, query):
        return self.GENERAL_SEARCH.format( s=urllib.parse.quote_plus(query) )

    def get_comments_before_comments_id_by_code(self, variables):
        return self.COMMENTS_BEFORE_COMMENT_ID_BY_CODE.format( s=urllib.parse.quote_plus(json.dumps(variables, separators=(',', ':'))) )

    def get_last_likes_by_code_old(self, code, count, last_like_id):
        return self.LIKES_BY_SHORTCODE_OLD.format( s=urllib.parse.quote_plus(code), s2=urllib.parse.quote_plus(str(count)), s3=urllib.parse.quote_plus(str(last_like_id)) )

    def get_last_likes_by_code(self, variables):
        return self.LIKES_BY_SHORTCODE.format( s=urllib.parse.quote_plus(json.dumps(variables, separators=(',', ':'))) )

    def get_follow_url(self, account_id):
        return self.FOLLOW_URL.format( s=urllib.parse.quote_plus(account_id) )

    def get_unfollow_url(self, account_id):
        return self.UNFOLLOW_URL.format( s=urllib.parse.quote_plus(account_id) )

    def get_followers_json_link_old(self, account_id, count, after=''):
        url = self.FOLLOWERS_URL_OLD.replace(
            '{{accountId}}', urllib.parse.quote_plus(account_id))
        url = url.replace('{{count}}', urllib.parse.quote_plus(str(count)))

        if after == '':
            url = url.replace('&after={{after}}', '')
        else:
            url = url.replace('{{after}}', urllib.parse.quote_plus(str(after)))

        return url

    def get_followers_json_link(self, variables):
        return self.FOLLOWERS_URL.format( s=urllib.parse.quote_plus(json.dumps(variables, separators=(',', ':'))) )

    def get_following_json_link_old(self, account_id, count, after=''):
        url = self.FOLLOWING_URL_OLD.replace(
            '{{accountId}}', urllib.parse.quote_plus(account_id))
        url = url.replace('{{count}}', urllib.parse.quote_plus(count))

        if after == '':
            url = url.replace('&after={{after}}', '')
        else:
            url = url.replace('{{after}}', urllib.parse.quote_plus(after))

        return url

    def get_following_json_link(self, variables):
        return self.FOLLOWING_URL.format( s=urllib.parse.quote_plus(json.dumps(variables, separators=(',', ':'))) )

    def get_user_stories_link(self, ):
        return self.get_graph_ql_url(self.USER_STORIES, {'variables': json.dumps([], separators=(',', ':'))})

    def get_graph_ql_url(self, query_id, parameters):
        url = self.GRAPH_QL_QUERY_URL.format( s=urllib.parse.quote_plus(query_id) )

        if len(parameters) > 0:
            query_string = urllib.parse.urlencode(parameters)
            url += '&' + query_string

        return url

    def get_stories_link(self, variables):
        return self.get_graph_ql_url(self.STORIES, {'variables': json.dumps(variables, separators=(',', ':'))})

    def get_like_url(self, media_id):
        return self.LIKE_URL.format( s=urllib.parse.quote_plus(str(media_id)) )

    def get_unlike_url(self, media_id):
        return self.UNLIKE_URL.format( s=urllib.parse.quote_plus(str(media_id)) )

    def get_add_comment_url(self, media_id):
        return self.ADD_COMMENT_URL.format( s=urllib.parse.quote_plus(str(media_id)) )

    def get_delete_comment_url(self, media_id, comment_id):
        return self.DELETE_COMMENT_URL.format( s=urllib.parse.quote_plus(str(media_id)), s2=urllib.parse.quote_plus(str(comment_id)) )

class TwoStepVerificationAbstractClass(__ABC):

    @abstractmethod
    def get_verification_type(self, possible_values):
        """
        :param possible_values: array of possible values
        :return: string
        """
        pass

    @abstractmethod
    def get_security_code(self):
        """

        :return: string
        """
        pass

class TwoStepConsoleVerification(TwoStepVerificationAbstractClass):

    def get_verification_type(self, choices):
        if (len(choices) > 1):
            possible_values = {}
            print('Select where to send security code')

            for choice in choices:
                print(choice['label'] + ' - ' + str(choice['value']))
                possible_values[str(choice['value'])] = True

            selected_choice = None

            while (not selected_choice in possible_values.keys()):
                if (selected_choice):
                    print('Wrong choice. Try again')

                selected_choice = input('Your choice: ').strip()
        else:
            print('Message with security code sent to: ' + choices[0]['label'])
            selected_choice = choices[0]['value']

        return selected_choice

    def get_security_code(self):
        """

        :return: string
        """
        security_code = ''
        while (len(security_code) != 6 and not security_code.isdigit()):
            if (security_code):
                print('Wrong security code')

            security_code = input('Enter security code: ').strip()

        return security_code

class Ticker(__threading.Thread):
  """A very simple thread that merely blocks for :attr:`interval` and sets a
  :class:`__threading.Event` when the :attr:`interval` has elapsed. It then waits
  for the caller to unset this event before looping again.

  Example use::

    t = Ticker(1.0) # make a ticker
    t.start() # start the ticker in a new thread
    try:
      while t.evt.wait(): # hang out til the time has elapsed
        t.evt.clear() # tell the ticker to loop again
        print time.time(), "FIRING!"
    except:
      t.stop() # tell the thread to stop
      t.join() # wait til the thread actually dies

  """
  # SIGALRM based timing proved to be unreliable on various python installs,
  # so we use a simple thread that blocks on sleep and sets a __threading.Event
  # when the timer expires, it does this forever.
  def __init__(self, interval):
    super(Ticker, self).__init__()
    self.interval = interval
    self.evt = __threading.Event()
    self.evt.clear()
    self.should_run = __threading.Event()
    self.should_run.set()

  def stop(self):
    """Stop the this thread. You probably want to call :meth:`join` immediately
    afterwards
    """
    self.should_run.clear()

  def consume(self):
    was_set = self.evt.is_set()
    if was_set:
      self.evt.clear()
    return was_set

  def run(self):
    """The internal main method of this thread. Block for :attr:`interval`
    seconds before setting :attr:`Ticker.evt`

    .. warning::
      Do not call this directly!  Instead call :meth:`start`.
    """
    while self.should_run.is_set():
      time.sleep(self.interval)
      self.evt.set()

# Telegram Bot

class TelegramBotCoreHT():

    @staticmethod
    def run():
        from . import TelegramBot


# Pool Blockchain
class Transaction:

    def __init__(self, sender_address, sender_private_key, recipient_address, value):
        self.sender_address = sender_address
        self.sender_private_key = sender_private_key
        self.recipient_address = recipient_address
        self.value = value

    def __getattr__(self, attr):
        return self.data[attr]

    def to_dict(self):
        return OrderedDict({'sender_address': self.sender_address,'recipient_address': self.recipient_address,'value': self.value})

    def sign_transaction(self):
        """
        Sign transaction with private key
        """
        private_key = RSA.importKey(binascii.unhexlify(self.sender_private_key))
        signer = PKCS1_v1_5.new(private_key)
        h = SHA.new(str(self.to_dict()).encode('utf8'))
        return binascii.hexlify(signer.sign(h)).decode('ascii')


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os

import time
import requests
import datetime
from bs4 import BeautifulSoup
from io import StringIO
import gzip

config = Config.getConfig(parentKey='modules', key='ht_pastebin')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = 'search_pastebin'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_pastebin'), debug_module=True)

	def search_pastebin(self, keywords, run_time=0, match_total=None, crawl_total=20):
		length = 0
		time_out = False
		found_keywords = []
		paste_list = set([])
		root_url = 'http://pastebin.com'
		raw_url = 'http://pastebin.com/raw/'
		start_time = datetime.datetime.now()

		keywords = ['ssh', 'pass', 'key', 'token']

		try:
			keywords = set(keywords.split(","))
		except:
			pass
		
		try:
			run_time = int(run_time)
		except:
			pass
		
		try:
			match_total = int(match_total)
		except:
			pass
		
		try:
			crawl_total = int(crawl_total)
		except:
			pass
		
		try:
			# Continually loop until user stops execution
			while True:

				#	Get pastebin home page html
				root_html = BeautifulSoup(self.__fetch_page__(root_url), 'html.parser')
				
				#	For each paste in the public pastes section of home page
				for paste in self.__find_new_pastes__(root_html):
					
					#	look at length of paste_list prior to new element
					length = len(paste_list)
					paste_list.add(paste)

					#	If the length has increased the paste is unique since a set has no duplicate entries
					if len(paste_list) > length:
						
						#	Add the pastes url to found_keywords if it contains keywords
						raw_paste = '{r}{p}'.format(r=str(raw_url), p=str(paste))
						found_keywords = self.__find_keywords__(raw_paste, found_keywords, keywords)

					else:

						#	If keywords are not found enter time_out
						time_out = True

				# Enter the timeout if no new pastes have been found
				if time_out:
					time.sleep(2)

				Logger.printMessage("Crawled total of {n} Pastes, Keyword matches {k}".format(n=len(paste_list), k=len(found_keywords)), debug_module=True)

				if run_time and (start_time + datetime.timedelta(seconds=run_time)) < datetime.datetime.now():
					Logger.printMessage("Reached time limit, Found {n} matches.".format(n=len(found_keywords)), debug_module=True)
					return found_keywords
				# Exit if surpassed specified match timeout 
				if match_total and len(found_keywords) >= match_total:
					Logger.printMessage("Reached match limit, Found {n} matches.".format(n=len(found_keywords)), debug_module=True)
					return found_keywords

				# Exit if surpassed specified crawl total timeout 
				if crawl_total and len(paste_list) >= crawl_total:
					Logger.printMessage("Reached total crawled Pastes limit, Found {n} matches.".format(n=len(found_keywords)), debug_module=True)
					return found_keywords

		# 	On keyboard interupt
		except KeyboardInterrupt:
			return found_keywords
		#	If http request returns an error and 
		except Exception as e:
			Logger.printMessage(str(e), is_error=True)
			return found_keywords

		return found_keywords

	def __find_new_pastes__(self, root_html):
		new_pastes = []

		div = root_html.find('div', {'id': 'menu_2'})
		ul = div.find('ul', {'class': 'right_menu'})
		
		for li in ul.findChildren():
			if li.find('a'):
				new_pastes.append(str(li.find('a').get('href')).replace("/", ""))

		return new_pastes

	def __find_keywords__(self, raw_url, found_keywords, keywords):
		paste = self.__fetch_page__(raw_url)

		#	Todo: Add in functionality to rank hit based on how many of the keywords it contains
		for keyword in keywords:
			if paste.find(keyword) != -1:
				found_keywords.append(raw_url)
				break

		return found_keywords

	def __fetch_page__(self, page):
		response = requests.get(page)
		return response.content.decode()


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os

import codecs
import itertools
import json
import re
import time
import warnings
from datetime import datetime
from urllib import parse as urlparse

from requests import RequestException
from requests_html import HTML, HTMLSession

config = Config.getConfig(parentKey='modules', key='ht_facebook')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

__base_url__ = 'https://m.facebook.com'
__user_agent__ = ("Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
               "AppleWebKit/537.36 (KHTML, like Gecko) "
               "Chrome/76.0.3809.87 Safari/537.36")
__headers__ = {'User-Agent': __user_agent__, 'Accept-Language': 'en-US,en;q=0.5'}

__session__ = None
__timeout__ = None

__likes_regex__ = re.compile(r'like_def[^>]*>([0-9,.]+)')
__comments_regex__ = re.compile(r'cmt_def[^>]*>([0-9,.]+)')
__shares_regex__ = re.compile(r'([0-9,.]+)\s+Shares')
__link_regex__ = re.compile(r"href=\"https:\/\/lm\.facebook\.com\/l\.php\?u=(.+?)\&amp;h=")

__cursor_regex__ = re.compile(r'href:"(/page_content[^"]+)"')  # First request
__cursor_regex_2__ = re.compile(r'href":"(\\/page_content[^"]+)"')  # Other requests

__photo_link__ = re.compile(r"href=\"(/[^\"]+/photos/[^\"]+?)\"")
__image_regex__ = re.compile(r"<a href=\"([^\"]+?)\" target=\"_blank\" class=\"sec\">View Full Size<\/a>")
__image_regex_lq__ = re.compile(r"background-image: url\('(.+)'\)")
__post_url_regex__ = re.compile(r'/story.php\?story_fbid=')

class StartModule():

	def __init__(self):
		self._main_gui_func_ = 'get_posts'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_facebook'), debug_module=True)

	def get_posts(self, account=None, group=None, pages=10, timeout=5, sleep=0, credentials=None):
		valid_args = sum(arg is not None for arg in (account, group))
		if valid_args != 1:
			raise ValueError("You need to specify either account or group")

		if account is not None:
			path = f'{account}/posts/'
		elif group is not None:
			path = f'groups/{group}/'

		return self.__get_posts__(path, pages=pages, timeout=timeout, sleep=sleep, credentials=credentials)

	def __get_posts__(self, path, pages=10, timeout=5, sleep=0, credentials=None):
		"""Gets posts for a given account."""
		global __session__, __timeout__

		url = f'{__base_url__}/{path}'

		__session__ = HTMLSession()
		__session__.headers.update(__headers__)

		if credentials:
			self.__login_user__(*credentials)

		__timeout__ = timeout
		response = __session__.get(url, timeout=__timeout__)
		html = response.html
		cursor_blob = html.html

		articles = []
		while True:
			for article in html.find('article'):
				articles.append(self.__extract_post__(article))

			pages -= 1
			if pages == 0:
				return articles

			cursor =  self.__find_cursor__(cursor_blob)
			next_url = f'{__base_url__}{cursor}'

			if sleep:
				time.sleep(sleep)

			try:
				response = __session__.get(next_url, timeout=timeout)
				response.raise_for_status()
				data = json.loads(response.text.replace('for (;;);', '', 1))
			except (RequestException, ValueError):
				return articles

			for action in data['payload']['actions']:
				if action['cmd'] == 'replace':
					html = HTML(html=action['html'], url=__base_url__)
				elif action['cmd'] == 'script':
					cursor_blob = action['code']

	def __extract_post__(self, article):
		text, post_text, shared_text =  self.__extract_text__(article)
		return {
			'post_id':  self.__extract_post_id__(article),
			'text': text,
			'post_text': post_text,
			'shared_text': shared_text,
			'time':  self.__extract_time__(article),
			'image':  self.__extract_image__(article),
			'likes':  self.__find_and_search__(article, 'footer', __likes_regex__,  self.__parse_int__) or 0,
			'comments':  self.__find_and_search__(article, 'footer', __comments_regex__,  self.__parse_int__) or 0,
			'shares':   self.__find_and_search__(article, 'footer', __shares_regex__,  self.__parse_int__) or 0,
			'post_url':  self.__extract_post_url__(article),
			'link':  self.__extract_link__(article),
		}

	def __extract_post_id__(self, article):
		try:
			data_ft = json.loads(article.attrs['data-ft'])
			return data_ft['mf_story_key']
		except (KeyError, ValueError):
			return None

	def __extract_text__(self, article):
		nodes = article.find('p, header')
		if nodes:
			post_text = []
			shared_text = []
			ended = False
			for node in nodes[1:]:
				if node.tag == "header":
					ended = True
				if not ended:
					post_text.append(node.text)
				else:
					shared_text.append(node.text)

			text = '\n'.join(itertools.chain(post_text, shared_text))
			post_text = '\n'.join(post_text)
			shared_text = '\n'.join(shared_text)

			return text, post_text, shared_text

		return None

	def __extract_time__(self, article):
		try:
			data_ft = json.loads(article.attrs['data-ft'])
			page_insights = data_ft['page_insights']
		except (KeyError, ValueError):
			return None

		for page in page_insights.values():
			try:
				timestamp = page['post_context']['publish_time']
				return datetime.fromtimestamp(timestamp)
			except (KeyError, ValueError):
				continue
		return None

	def __extract_photo_link__(self, article):
		match = __photo_link__.search(article.html)
		if not match:
			return None

		url = f"{__base_url__}{match.groups()[0]}"

		response = __session__.get(url, timeout=__timeout__)
		html = response.html.html
		match = __image_regex__.search(html)
		if match:
			return match.groups()[0].replace("&amp;", "&")
		return None

	def __extract_image__(self, article):
		image_link =  self.__extract_photo_link__(article)
		if image_link is not None:
			return image_link
		return  self.__extract_image_lq__(article)

	def __extract_image_lq__(self, article):
		story_container = article.find('div.story_body_container', first=True)
		other_containers = story_container.xpath('div/div')

		for container in other_containers:
			image_container = container.find('.img', first=True)
			if image_container is None:
				continue

			style = image_container.attrs.get('style', '')
			match = __image_regex_lq__.search(style)
			if match:
				return  self.__decode_css_url__(match.groups()[0])

		return None

	def __extract_link__(self, article):
		html = article.html
		match = __link_regex__.search(html)
		if match:
			return urlparse.unquote(match.groups()[0])
		return None

	def __extract_post_url__(self, article):
		query_params = ('story_fbid', 'id')

		elements = article.find('header a')
		for element in elements:
			href = element.attrs.get('href', '')
			match = __post_url_regex__.match(href)
			if match:
				path =  self.__filter_query_params__(href, whitelist=query_params)
				return f'{__base_url__}{path}'

		return None

	def __find_and_search__(self, article, selector, pattern, cast=str):
		container = article.find(selector, first=True)
		match = container and pattern.search(container.html)
		return match and cast(match.groups()[0])

	def __find_cursor__(self, text):
		match = __cursor_regex__.search(text)
		if match:
			return match.groups()[0]

		match = __cursor_regex_2__.search(text)
		if match:
			value = match.groups()[0]
			return value.encode('utf-8').decode('unicode_escape').replace('\\/', '/')

		return None

	def __parse_int__(self, value):
		return int(''.join(filter(lambda c: c.isdigit(), value)))

	def __decode_css_url__(self, url):
		url = re.sub(r'\\(..) ', r'\\x\g<1>', url)
		url, _ = codecs.unicode_escape_decode(url)
		return url

	def __filter_query_params__(self, url, whitelist=None, blacklist=None):
		def is_valid_param(param):
			if whitelist is not None:
				return param in whitelist
			if blacklist is not None:
				return param not in blacklist
			return True  # Do nothing

		parsed_url = urlparse.urlparse(url)
		query_params = urlparse.parse_qsl(parsed_url.query)
		query_string = urlparse.urlencode(
			[(k, v) for k, v in query_params if is_valid_param(k)]
		)
		return urlparse.urlunparse(parsed_url._replace(query=query_string))

	def __login_user__(self, email, password):
		login_page = __session__.get(__base_url__)
		login_action = login_page.html.find('#login_form', first=True).attrs.get('action')
		__session__.post(__base_url__ + login_action, data={'email': email, 'pass': password})
		if 'c_user' not in __session__.cookies:
			warnings.warn('login unsuccessful')


try:
    import telebot as __telebot
    from telebot import types
    import time as __time
    import hackingtools as ht
    import ast as __ast
    
    __bot_name__ = ht.core.Config.getConfig(parentKey='core', key='TelegramBot', subkey='bot-name')
    __bot_token__ = ht.core.Config.getConfig(parentKey='core', key='TelegramBot', subkey='bot-token')
    bot = None

    __crossIcon__ = u"\u274C"

    try:
        bot = __telebot.TeleBot(__bot_token__)
    except:
        pass

    @bot.message_handler(commands=['start'])
    def handle_command_adminwindow(message):
        modules = ht.getModulesNames()

        markup = types.InlineKeyboardMarkup()

        modules_list_3 = ht.Utils.groupListByLength(modules, 3) 
        
        # [ 
        #     ['ht_shodan','ht_*','ht_*'],
        #     ['ht_*','ht_*','ht_*'],
        #     ['ht_*','ht_*','ht_*'] ,
        #     ['ht_*','ht_*'] 
        # ]

        # ['value', 'ht_shodan', 'osint']

        for line in modules_list_3:
            if len(line) == 3:
                markup.add(types.InlineKeyboardButton(text=line[0], callback_data="['value', '" + line[0] + "', '" + ht.getModuleCategory(line[0]) + "']"),
                types.InlineKeyboardButton(text=line[1], callback_data="['value', '" + line[1] + "', '" + ht.getModuleCategory(line[1]) + "']"),
                types.InlineKeyboardButton(text=line[2], callback_data="['value', '" + line[2] + "', '" + ht.getModuleCategory(line[2]) + "']"))
            elif len(line) == 3:
                markup.add(types.InlineKeyboardButton(text=line[0], callback_data="['value', '" + line[0] + "', '" + ht.getModuleCategory(line[0]) + "']"),
                types.InlineKeyboardButton(text=line[1], callback_data="['value', '" + line[1] + "', '" + ht.getModuleCategory(line[1]) + "']"))
            else:
                markup.add(types.InlineKeyboardButton(text=line[0], callback_data="['value', '" + line[0] + "', '" + ht.getModuleCategory(line[0]) + "']"))

        bot.send_message(chat_id=message.chat.id, text="Here you have the modules:", reply_markup=markup)
        
    @bot.callback_query_handler(lambda q: q.message.chat.type == "private")
    def private_query(query):
        try:
            if (query.data.startswith("['value'")):
                moduleName = __ast.literal_eval(query.data)[1]
                category = __ast.literal_eval(query.data)[2]

                functions = '\n'.join( ht.getFunctionsNamesFromModule(moduleName) )

                bot.answer_callback_query(callback_query_id=query.id, show_alert=True, text=functions)
                
            bot.edit_message_reply_markup(query.message.chat.id, query.message.message_id)
        except Exception as e:
            bot.answer_callback_query(callback_query_id=query.id, show_alert=True, text=str(e))

    bot.polling()
except:
    try:
        bot.stop_polling()
    except:
        pass


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os, sys, requests, json

config = Config.getConfig(parentKey='modules', key='ht_finder')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = 'checkerOnline'
		self.options = {
			'facebook' : self.__facebook__,
			'twitter' : self.__twitter__,
			'instagram' : self.__instagram__,
			'medium' : self.__medium__,
			'github' : self.__github__,
			'bitbucket' : self.__bitbucket__,
		}

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_finder'), debug_module=True)

	def findAll(self, username):
		data = {}
		for opt, func in self.options.items():
			value = func(username)
			if value:
				data[opt] = value
		if data:
			return data
		return None

	def checkerOnline(self, username):
		url = 'https://checkuser.org/existence.php'
		posibilities = {}
		for i in range(1, 52): # There are 52 posibilities in the website
			data = {
				'index' : i,
				'username' : username
			}
			response = json.loads(requests.post(url, data=data).text)
			if response['status'] != 400:
				posibilities[ response['domain'].split('.')[0] ] = response['url']
		
		if posibilities:
			return posibilities
		return None			

	# a function for each site, which creates the request according to the username
	def __bitbucket__(self, username):
		url = "https://bitbucket.org/" + username + "/"
		request = requests.get(url)
		data = self.__parse__(request, username, url)
		return data

	def __github__(self, username):
		url = "https://github.com/" + username + "/"
		request = requests.get(url)
		data = self.__parse__(request, username, url)
		return data

	def __twitter__(self, username):
		url = "https://twitter.com/" + username + "/"
		request = requests.get(url)
		data = self.__parse__(request, username, url)
		return data

	def __instagram__(self, username):
		url = "https://instagram.com/" + username + "/"
		request = requests.get(url)
		data = self.__parse__(request, username, url)
		return data

	def __medium__(self, username):
		url = "https://medium.com/@" + username + "/"
		# Medium requires a browser as a user agent, such as chrome
		headers = {
			"User-Agent" : "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2490.86 Safari/537.36"
		}
		request = requests.get(url, headers)
		data = self.__parse__(request, username, url)
		return data
		
	def __facebook__(self, username):
		url = "http://facebook.com/" + username + "/"
		request = requests.get(url)
		data = self.__parse__(request, username, url)
		return data

	# used for generating colored responses
	def __parse__(self, request, username, url):
		if request.status_code == 200:
			return url
		return None

	


from hackingtools.core import Logger, Utils, Config, Connections
import hackingtools as ht
import os

from bs4 import BeautifulSoup
import requests

config = Config.getConfig(parentKey='modules', key='ht_cve')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = 'searchCVE'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_cve'), debug_module=True)

	def searchCVE(self, cve_id):
		try:
			url = 'https://www.cvedetails.com/cve/{id}'.format(id=cve_id.strip())
			response = requests.get(url, headers=Connections.__headers__)
			soup = BeautifulSoup(response.text,"html.parser")

			data = {}
			vulnered_systems = []
			metasploit_modules = []
			table_heading_list = []

			data['CVE'] = soup.find(attrs={"name":"description"})['content'].split(':', 1)[0].lstrip().rstrip().strip()

			data['Description'] = soup.find(attrs={"name":"description"})['content'].split(':', 1)[1].strip()

			data['CVSS'] = soup.find('div', attrs={'class' : 'cvssbox'}).contents[0]

			heading_list = ["Confidentiality Impact", "Integrity Impact", "Availability Impact", "Access Complexity", "Authentication", "Gained Access", "Vulnerability Type(s)", "CWE ID"]
			for h in heading_list:
				for heading in soup.find_all("th"):
					if h == heading.text:
						data[h] = soup.find(text=h).findNext('td').text.replace('\n', ' ').strip()


			table_vulnered_systems = soup.find("table", {"id": "vulnprodstable"})
			if table_vulnered_systems:
				table_rows = table_vulnered_systems.find_all("tr")

				for h in table_rows[0].find_all("td"):
					table_heading_list.append(h.text.replace('\n', ' ').strip())

				del table_rows[0]

				for row in table_rows:
					system_found = ''
					for index_c, column in enumerate(row.find_all("td")):
						if index_c == 2:
							system_found = column.text.lstrip().rstrip().strip()
						if index_c == 3:
							system_found = '{m} {mm}'.format(m=system_found, mm=column.text.lstrip().rstrip().strip())
						if index_c == 4:
							system_found = '{m} {mm}'.format(m=system_found, mm=column.text.lstrip().rstrip().strip())
						if index_c == 5:
							system_found = '{m} {mm}'.format(m=system_found, mm=column.text.lstrip().rstrip().strip())
						if index_c == 6:
							system_found = '{m} {mm}'.format(m=system_found, mm=column.text.lstrip().rstrip().strip())
					if system_found:
						vulnered_systems.append(system_found.lstrip().rstrip().strip())

				if vulnered_systems:
					data['Products Affected'] = vulnered_systems

			table_metasploit_modules = soup.find("table", {"class": "metasploit"})
			if table_metasploit_modules:
				table_rows = table_metasploit_modules.find_all("tr")
				table_rows_grouped = ht.Utils.groupListByLength(table_rows, 2)

				for t_group in table_rows_grouped:
					d = {}
					d['link'] = t_group[0].find('a', href=True)['href'].lstrip().rstrip().strip()
					d['name'] = t_group[0].find('a').text.lstrip().rstrip().strip()
					d['description'] = t_group[1].text.lstrip().rstrip().strip()
					metasploit_modules.append(d)

				if metasploit_modules:
					data['Metasploit Modules Related'] = metasploit_modules

			return data
		except Exception as e:
			return str(e)




from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os
import r2pipe

config = Config.getConfig(parentKey='modules', key='ht_radare2')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():


	def __init__(self):
		self._main_gui_func_ = ''

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_radare2'), debug_module=True)

	def getImports(self, filemon):
		try:
			r = r2pipe.open(filemon)
			# saca la tabla de dll importadas en json
			return r.cmd('iij')
		except Exception as e:
			return str(e)


from . import Config, Logger, Utils, Pool

config = Config.getConfig(parentKey='core', key='Connections')
import sys, requests, socket, os

# Connections Treatment
global __services__
__services__ = []

global __listening_port__
__listening_port__ = '8000'

global __https__
__https__ = ''

__headers__ = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36'
}

__tor_proxies__ = {
    'http': 'socks5://127.0.0.1:9150',
    'https': 'socks5://127.0.0.1:9150'
}

# Ngrok connections
global __ngrok_ip__
__ngrok_ip__ = None

# Main functions
def getMyServices():
    global __services__
    return [serv for serv in __services__ if serv]

def addMineService(serv):
    global __services__
    Logger.printMessage('Adding service', serv, debug_core=True)
    if not serv in __services__:
        __services__.append(serv)

def getActualPort():
    global __listening_port__
    return __listening_port__

def getMyPublicIP(as_service=False):
    global __listening_port__
    global __https__
    try:
        if as_service:
            return Logger.print_and_return(msg='getMyPublicIP', value='http{s}://{i}:{p}'.format(s=__https__,
                                                                                                 i=requests.get(
                                                                                                     '__https__://api.ipify.org').text,
                                                                                                 p=__listening_port__),
                                           debug_core=True)
        return Logger.print_and_return(msg='getMyPublicIP', value=requests.get('__https__://api.ipify.org').text,
                                       debug_core=True)
    except:
        if as_service:
            return Logger.print_and_return(msg='getMyPublicIP',
                                           value='http{s}://127.0.0.1:{p}'.format(s=__https__, p=__listening_port__),
                                           debug_core=True)
        return Logger.print_and_return(msg='getMyPublicIP', value='127.0.0.1', debug_core=True)

def getMyLanIP(as_service=False):
    global __listening_port__
    global __https__
    for ip in socket.gethostbyname_ex(socket.gethostname())[-1]:
        if '192.168.1' in ip or '192.168.0' in ip:
            if as_service:
                return Logger.print_and_return(msg='getMyLanIP', value='http{s}://{i}:{p}'.format(s=__https__, i=ip,
                                                                                                  p=__listening_port__),
                                               debug_core=True)
            return Logger.print_and_return(msg='getMyLanIP', value=ip, debug_core=True)

def getMyLocalIP(as_service=False, port=True):
    global __services__
    global __listening_port__
    global __https__
    if isHeroku():
        if not __services__:
            Pool.__checkPoolNodes__()
            Config.__readConfig__()
            __services__ += Config.getConfig(parentKey='core', key='Connections', subkey='my___services__')
        if as_service:
            if port:
                return Logger.print_and_return(msg='getMyLocalIP',
                                               value='http{s}://{ss}:{p}'.format(s=__https__, ss=__services__[0],
                                                                                 p=__listening_port__), debug_core=True)
            return Logger.print_and_return(msg='getMyLocalIP', value=__services__[0], debug_core=True)
        return Logger.print_and_return(msg='getMyLocalIP', value=__services__[0], debug_core=True)
    if as_service:
        if port:
            return Logger.print_and_return(msg='getMyLocalIP',
                                           value='http{s}://127.0.0.1:{p}'.format(s=__https__, p=__listening_port__),
                                           debug_core=True)
        return Logger.print_and_return(msg='getMyLocalIP', value='http{s}://127.0.0.1'.format(s=__https__),
                                       debug_core=True)
    return Logger.print_and_return(msg='getMyLocalIP', value='127.0.0.1', debug_core=True)

def isHeroku():
    if 'DYNO' in os.environ:  # Automatically Heroku Deploy
        return True
    return False

def getNgrokServiceUrl():
    global __ngrok_ip__
    return __ngrok_ip__

def startNgrok(port=__listening_port__):
    try:
        from pyngrok import ngrok
        global __ngrok_ip__
        __ngrok_ip__ = ngrok.connect(int(port))
        if __ngrok_ip__:
            __services__.append(__ngrok_ip__)
            return Logger.print_and_return(msg='ngrok', value=__ngrok_ip__)
    except Exception as e:
        Logger.printMessage(message='Couldn\'t start ngrok service', description=str(e), is_error=True)
        return None

def stopNgrok(ngrokServiceUrl):
    try:
        from pyngrok import ngrok
        global __ngrok_ip__
        ngrok.disconnect(ngrokServiceUrl)
        __services__.remove(ngrokServiceUrl)
        return True
    except Exception as e:
        Logger.printMessage(message='Couldn\'t stop ngrok service', description=str(e), is_error=True)
        return False

def requestByTor(url):
    try:
        return requests.get(url, proxies=__tor_proxies__).text.strip()
    except Exception as e:
        Logger.printMessage( str(e), is_error=True)
        return None

def __serviceNotMine__(service):
    for serv in __services__:
        if service == serv:
            return False
    return True

def __loadActualPort__():
    global __listening_port__
    try:
        __listening_port__ = Logger.print_and_return(msg='__loadActualPort__', value=sys.argv[-1].split(':')[1],
                                                     debug_core=True)
    except:
        __listening_port__ = Logger.print_and_return(msg='__loadActualPort__', value='8000', debug_core=True)

def __initServices__():
    if not isHeroku():
        global __services__
        global __https__

        __loadActualPort__()

        if 'ngrok' in config and config['ngrok'] and '__load_on_boot__' in config['ngrok'] and config['ngrok'][
            '__load_on_boot__'] == True:
            startNgrok(getActualPort())

        # for service in (getMyPublicIP(), getMyLanIP(), getMyLocalIP()):
        #     __services__.append('http{s}://{ip}:{port}'.format(s=__https__, ip=service, port=getActualPort()))

        global __ngrok_ip__
        if __ngrok_ip__:
            __services__ = [__ngrok_ip__]
        else:
            [__services__.append(service) for service in (getMyLanIP(as_service=True), getMyLocalIP(as_service=True))]
    else:
        Pool.__checkPoolNodes__()
        Config.__readConfig__()
        __services__ += Config.getConfig(parentKey='core', key='Connections', subkey='my___services__')
    Logger.printMessage(message='Loaded __services__', description=__services__, color=Logger.__Fore.YELLOW,
                        debug_core=True)

# Init
if not isHeroku():
    __initServices__()



from hackingtools.core import Logger, Config, Utils
import hackingtools as ht

import random
import base64
import os

config = Config.getConfig(parentKey='modules', key='ht_rsa')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():
	'''How to use:

	prime_a, prime_b = self.getRandomKeypair()
	public, private = self.generate_keypair(prime_a, prime_b)

	print('Public key: {pub} - Private Key: {pri}'.format(pub=public, pri=private))

	encrypted_msg = self.encrypt(private, message)
	print('Message ciphered: {msg}'.format(msg=encrypted_msg))

	decrypted_msg = self.decrypt(public, encrypted_msg)
	print('Message deciphered: {msg}'.format(msg=decrypted_msg))
	
	'''

	def __init__(self):
		self._main_gui_func_ = 'encode'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_rsa'), debug_module=True)

	def generate_keypair(self, prime_a, prime_b):
		if not (Utils.isPrime(int(prime_a)) and Utils.isPrime(int(prime_b))):
			Logger.printMessage(message='{methodName}'.format(methodName='generate_keypair'), description=config['bad_identical_prime'], debug_module=True, is_error=True)
			return config['bad_identical_prime']
		elif prime_a == prime_b:
			Logger.printMessage(message='{methodName}'.format(methodName='generate_keypair'), description=config['p_q_equal_error'], debug_module=True, is_error=True)
			return config['p_q_equal_error']
		else:
			#n = pq
			n = prime_a * prime_b

			#Phi is the totient of n
			phi = (prime_a-1) * (prime_b-1)

			#Choose an integer e such that e and phi(n) are coprime
			e = random.randrange(1, phi)

			#Use Euclid's Algorithm to verify that e and phi(n) are comprime
			g = Utils.euclides(e, phi)
			while g != 1:
				e = random.randrange(1, phi)
				g = Utils.euclides(e, phi)

			#Use Extended Euclid's Algorithm to generate the private key
			d = Utils.multiplicativeInverse(e, phi)
			
			#Return public and private keypair
			#Public key is (e, n) and private key is (d, n)
			return ((e, n), (d, n))

	def getRandomKeypair(self, length = 8):
		Logger.printMessage(message='{methodName}'.format(methodName='getRandomKeypair'), debug_module=True)
		prime_a = ''
		prime_b = ''
		while prime_a == prime_b:
			while prime_a == '' or prime_a == prime_b:
				prime_a = Utils.getRandomPrimeByLength(length)
			while prime_b == '' or prime_a == prime_b:
				prime_b = Utils.getRandomPrimeByLength(length)
		if prime_a > prime_b:
			temp = prime_b
			prime_b = prime_a
			prime_a = temp
		Logger.printMessage(message='getRandomKeypair', description=(prime_a, prime_b), debug_module=True)
		return (prime_a, prime_b)
	
	def encrypt(self, private_key, plaintext):
		try:
			Logger.printMessage(message='{methodName}'.format(methodName='encrypt'), description='{private_key} - {msg}'.format(private_key=private_key, msg=plaintext[0:10]), debug_module=True)
			#Unpack the key into it's components
			key, n = private_key
			ba64 = base64.b64encode(plaintext)
			ashex = Utils.asciiToHex(ba64)
			hexba64 = Utils.hexToBase64(ashex)
			ba64un = Utils.joinBase64(hexba64)
			decasc = Utils.decimalToAscii(ba64un)
			mensaje = Utils.textToAscii(decasc)
			Logger.printMessage(message='{methodName}'.format(methodName='encrypt'), description='{msg} - Length: {l}'.format(msg=mensaje[0:10], l=len(mensaje)), debug_module=True)
			mensaje1 = [(ord(chr(char)) ** key) % n for char in mensaje]
			mensajeHex = Utils.asciiToHex(mensaje1)
			mensajeBase64 = Utils.hexToBase64(mensajeHex)
			mensajeFinalBase64 = Utils.joinBase64(mensajeBase64)
			return mensajeFinalBase64.decode("utf-8")
		except Exception as e:
			Logger.printMessage(message='{methodName}'.format(methodName='encrypt'), description='{msg}'.format(msg=e))
			return

	def decrypt(self, public_key, ciphertext):
		Logger.printMessage(message='{methodName}'.format(methodName='decrypt'), description='{public_key}'.format(public_key=public_key), debug_module=True)
		#Unpack the key into its components
		key, n = public_key
		menRec = Utils.asciiToBase64(ciphertext.encode('utf-8'))
		menHex = Utils.base64ToHex(menRec)
		menDec = Utils.hexToDecimal(menHex)
		Logger.printMessage(message='{methodName}'.format(methodName='decrypt'), description='{msg}'.format(msg=menDec[0:10]), debug_module=True)
		menDesc = [((char ** key) % n) for char in menDec]
		menAscii = Utils.decimalToAscii(menDesc)
		decasc = Utils.asciiToBase64(''.join(menAscii).encode())
		hexba64 = Utils.base64ToHex(decasc)
		ashex = Utils.hexToDecimal(hexba64)
		deasc = Utils.decimalToAscii(ashex)
		ba64 = base64.b64decode(deasc.encode())
		return ba64

	def encode(self, password, plaintext):
		enc = []
		for i in range(len(plaintext)):
			key_c = password[i % len(password)]
			enc_c = chr((ord(plaintext[i]) + ord(key_c)) % 256)
			enc.append(enc_c)
		return base64.urlsafe_b64encode("".join(enc).encode()).decode()

	def decode(self, password, ciphertext):
		dec = []
		ciphertext = base64.urlsafe_b64decode(ciphertext).decode()
		for i in range(len(ciphertext)):
			key_c = password[i % len(password)]
			dec_c = chr((256 + ord(ciphertext[i]) - ord(key_c)) % 256)
			dec.append(dec_c)
		return "".join(dec)

	def encodeFromComplexMap(self, password, plaintext):
		data = self.encode(password, plaintext)
		complexMap = [ ht.Utils.randomText(length=16, alphabet='mixalpha-numeric-symbol14') for i in range(16) ]

		data_rss_encoded = []
		for char in data:
			char_mapped = False
			rss_appearances = ''
			for line_index, line in enumerate(complexMap):
				if not char_mapped:
					appearances = [i for i in range(len(line)) if line.startswith(char, i)]
					for app in appearances:
						if not char_mapped:
							use_this_appearance = bool(random.getrandbits(1))
							if use_this_appearance:
								rss_appearances = '{l}{s}{a}{ss}'.format( l=line_index, a=app, s=ht.Utils.getRandomCharFromDict('symbols14'), ss=ht.Utils.getRandomCharFromDict('symbols14') )
								char_mapped = True

			if not char_mapped:
				print('No mapeado: ', char)
				rss_appearances = '{l}{s}{l}{ss}'.format( l=char, a=char, s=ht.Utils.getRandomCharFromDict('symbols14'), ss=ht.Utils.getRandomCharFromDict('symbols14') )

			data_rss_encoded.append(rss_appearances)

		return ( ''.join(data_rss_encoded), ''.join(complexMap) )

	def decodeFromComplexMap(self, password, ciphertext, complexMap):
		try:
			full_data = ciphertext

			symbols14 = Config.getConfig('core', 'Utils', 'dictionaries', 'symbols14')

			full_lines = []

			if not isinstance(complexMap, list):
				complexMap = ''.join(complexMap.split('\n'))
				chunks, chunk_size = len(complexMap), len(complexMap)//16
				complexMap = [ complexMap[i:i+chunk_size] for i in range(0, chunks, chunk_size) ]

			better_decoded = full_data
			for symb in symbols14:
				better_decoded = ' '.join(better_decoded.split(symb))

			list_decoded = better_decoded.split(' ')

			count = 0
			for row in range( int(len(list_decoded)/2) ):
				full_lines.append( [ list_decoded[row + count], list_decoded[row + count + 1] ] )
				count += 1

			rss_decoded = []

			for lines in full_lines:
				row_n = lines[0]
				char_n = lines[1]
				if row_n == '':
					row_n = -1
				if char_n == '':
					char_n = -1
				try:
					row_n = int(row_n)
				except:
					pass
				try:
					char_n = int(char_n)
				except:
					pass
				if not isinstance(row_n, int) or not isinstance(char_n, int):
					rss_decoded.append(row_n)
				else:
					rss_decoded.append(complexMap[row_n][char_n])

			return self.decode(password, ''.join(rss_decoded))
		except:
			return 'Bad password'


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os

import requests
from requests.auth import HTTPBasicAuth
from urllib.parse import urljoin

config = Config.getConfig(parentKey='modules', key='ht_qualys')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = ''
		
		self.server = ''

		self.session = None
		self.auth = None
		self.cookies = None

		self.username = None
		self.password = None

		self.urls = {}
		self.urls['US1'] = 'https://qualysguard.com'
		self.urls['US2'] = 'https://qualysguard.qg2.apps.qualys.com'
		self.urls['US3'] = 'https://qualysguard.qg3.apps.qualys.com'
		self.urls['US4'] = 'https://qualysguard.qg4.apps.qualys.com'
		self.urls['EU1'] = 'https://qualysguard.eu'
		self.urls['EU2'] = 'https://qualysguard.qg2.apps.qualys.eu'
		self.urls['IN1'] = 'https://qualysguard.qg1.apps.qualys.in'
		self.urls['CA1'] = 'https://qualysguard.qg1.apps.qualys.ca'

		self.headers = {} 
		self.headers['X-Requested-With'] = 'ht-qualys-connector'

		self.api = {}
		self.api['session'] = '/api/2.0/fo/session/'

		self.api['common'] = {}
		self.api['common']['assets'] = '/qps/rest/2.0/search/am/hostasset/'
		self.api['common']['assets2'] = '/api/2.0/fo/asset/host/'
		self.api['common']['scan'] = '/api/2.0/fo/scan/'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_qualys'), debug_module=True)

	def __getServerUrlByUsername__(self, username):
		try:
			if '_' in username:
				self.server = self.urls['US1']
			elif '2' in username:
				self.server = self.urls['US2']
			elif '3' in username:
				self.server = self.urls['US3']
			elif '4' in username:
				self.server = self.urls['US4']
			elif '-' in username:
				self.server = self.urls['EU1']
			elif '5' in username or '!' in username:
				self.server = self.urls['EU2']
			elif '8' in username:
				self.server = self.urls['IN1']
			elif '9' in username:
				self.server = self.urls['CA1']
			else:
				self.server = self.urls['US1']
		except:
			self.server = self.urls['US1']

	def connect(self, username, password):
		try:
			# Check and set the server depending on the user
			self.__getServerUrlByUsername__(username)

			# Save credentiales temporaly
			self.username = username
			self.password = password

			# Create payload for the API
			data = {
				'username' : self.username,
				'password' : self.password,
				'action' : 'login'
			}
			
			# Join safely the server URL with the API request path
			qualys_url = urljoin(self.server, self.api['session'])

			# Start a session for getting later the cookie that we will use for login next times
			self.session = requests.Session()

			# Get the response from the POST requests
			response = self.session.post( qualys_url, data=data, headers=self.headers, auth=self.auth )

			# If all is ok
			if response.status_code == 200:
				# Save the cookies for later sessions
				self.cookies = self.session.cookies.get_dict()
				self.auth = HTTPBasicAuth( self.username, self.password )
				return self
			return None
		except:
			return None

	def disconnect(self):
		try:
			if self.session:
				# Create payload for the API
				data = {
					'action' : 'logout'
				}
				
				# Join safely the server URL with the API request path
				qualys_url = urljoin(self.server, self.api['session'])

				# Get the response from the POST requests
				response = self.session.post( qualys_url, data=data, headers=self.headers, cookies=self.cookies, auth=self.auth )

				# If all is ok
				if response.status_code == 200:
					# Empty all session params
					self.cookies, self.username, self.password, self.session = ( None, None, None, None )
					return self
			return None
		except:
			return None

	def scan(self, ips):
		try:
			if self.session:
				# Create payload for the API
				data = {
					'ips' : ips,
					'action' : 'list'
				}

				# Join safely the server URL with the API request path
				qualys_url = urljoin(self.server, self.api['common']['assets2'])

				# Get the response from the POST requests
				response = self.session.get( qualys_url, data=data, headers=self.headers, cookies=self.cookies, auth=self.auth )

				# If all is ok
				if response.status_code == 200:
					return response.text
			return None
		except:
			return None



class IndexError(Exception):

    def __init__(self, message):
        super().__init__(message)

class ConnectionTimeout(Exception):

    def __init__(self, message):
        super().__init__(message)

class InvalidValue(Exception):

    def __init__(self, message):
        super().__init__(message)

class QueryError(Exception):

    def __init__(self, message):
        super().__init__(message)

class ParameterRequired(Exception):

    def __init__(self, message):
        super().__init__(message)

class NotFoundException(Exception):

    def __init__(self, message="", code=404):
        super().__init__(f'{message}, Code:{code}')

class AnyException(Exception):

    def __init__(self, message="", code=500):
        super().__init__(f'{message}, Code:{code}')
    
    @staticmethod
    def default(response_text, status_code):
        return AnyException('Response code is {status_code}. Body: {response_text}. Something went wrong. Please report issue.'.format(response_text=response_text, status_code=status_code), status_code)

class AuthException(Exception):
    def __init__(self, message = "", code = 401):
        super().__init__(f'{message}, Code:{code}')


from hackingtools.core import Logger, Config
import hackingtools as ht
import os, json

# Image Metadata
from PIL import Image
# PDF Metadata
from PyPDF2 import PdfFileReader, PdfFileWriter
import pdfrw
from pdfrw import PdfReader, PdfWriter
# MP3
import eyed3
# import mutagen

import os

config = Config.getConfig(parentKey='modules', key='ht_metadata')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = 'set_pdf_metadata_to_another'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_metadata'), debug_module=True)

	def get_image_exif(self, filename):
		Logger.printMessage(message='{methodName}'.format(methodName='get_image_exif'), description=filename, debug_module=True)
		try:
			img_file = Image.open(filename)
			img_file.verify()
			info = img_file._getexif()
			return info
		except Exception as e:
			Logger.printMessage(message='{methodName}'.format(methodName='exception'), description=e, debug_module=True)
			return e
		return -1

	def get_pdf_exif(self, pdf_file):
		Logger.printMessage(message='{methodName}'.format(methodName='get_pdf_exif'), description=pdf_file, debug_module=True)
		info = ''
		data = {}
		try:
			with open(pdf_file, 'rb') as f:
				pdf = PdfFileReader(f)
				info = pdf.getDocumentInfo()
				number_of_pages = pdf.getNumPages()
			if info:
				for a in info:
					data[a] = info[a]
			return data
		except Exception as e:
			Logger.printMessage(message='{methodName}'.format(methodName='exception'), description=e, debug_module=True)
			return e
		return -1
	
	def set_pdf_field_value(self, pdf_file, field, fieldValue):
		try:
			trailer = PdfReader(pdf_file)
			if trailer.Info is None:
				trailer.Info = pdfrw.objects.pdfdict.PdfDict()
			setDataQuery = 'trailer.Info.{f} = "{v}"'.format(f=field.replace('/', ''), v=fieldValue)
			exec(setDataQuery)
			new_pdf = os.path.split(pdf_file)[1]
			new_file = os.path.join( output_dir, new_pdf )
			PdfWriter(new_file, trailer=trailer).write()
			return new_file
		except:
			Logger.printMessage(pdf_file, is_error=True)
			return pdf_file

	def set_pdf_metadata_to_another(self, pdf_file_original, pdf_file_fake):
		meta = self.get_pdf_exif(pdf_file_original)
		for data, val in meta.items():
			pdf_file_fake = self.set_pdf_field_value(pdf_file_fake, data, val)
		return pdf_file_fake

	def get_mp3_exif(self, mp3_file):
		try:
			audioFile = eyed3.load(mp3_file)
			keys = [ tag for tag in dir(audioFile.tag) if not tag.startswith('_') ]
			data = {}
			for k in keys:
				temp = eval("audioFile.tag.{f}".format(f=k))
				if temp is not None and temp:
					if not isinstance(temp, str):
						subkeys = [ t for t in dir(temp) if not t.startswith('_') ]
						if subkeys:
							data[k] = {}
							for s in subkeys:
								new_temp = eval("audioFile.tag.{f}.{r}".format(f=k, r=s))
								if new_temp is not None and new_temp:
									if isinstance(new_temp, str):
										if str(new_temp) != 'None':
											data[k][s] = str(new_temp)
						if k in data and not data[k]:
							del data[k]
					else:
						data[k] = str(temp)
			return data
		except Exception as e:
			Logger.printMessage(str(e), is_error=True)
			return str(e)

	def set_mp3_metadata(self, mp3_file, title=None, artist=None, album=None, album_artist=None, track_num=None):
		try:
			new_file = "edited-{f}".format(f=os.path.split(mp3_file)[1])
			new_file = os.path.join(output_dir, new_file)
			audioFile = eyed3.load(mp3_file)
			if title:
				audioFile.tag.title = title
			if artist:
				audioFile.tag.artist = artist
			if album:
				audioFile.tag.album = album
			if album_artist:
				audioFile.tag.album_artist = album_artist
			if track_num:
				audioFile.tag.title = track_num
			audioFile.tag.save(new_file)
			return new_file
		except Exception as e:
			Logger.printMessage(str(e), is_error=True)
			return str(e)

	# def get_audio_video_exif(self, filename):
	# 	Logger.printMessage(message='{methodName}'.format(methodName='get_audio_video_exif'), description=filename, debug_module=True)
	# 	try:
	# 		my_file = mutagen.File(filename)
	# 		img_file.verify()
	# 		info = img_file._getexif()
	# 		return info
	# 	except Exception as e:
	# 		Logger.printMessage(message='{methodName}'.format(methodName='exception'), description=e, debug_module=True)
	# 		return e
	# 	return -1



from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os, requests

config = Config.getConfig(parentKey='modules', key='ht_vulndb')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = ''
		self.headers = {
			'X-VulDB-ApiKey' : ht.Config.getAPIKey('vulndb_api')
		}
		self.url = 'https://vulners.com/api/v3'
		self.options = {
			'suggest' : '/search/suggest/',
			'search-cve' : '/search/id/'
		}

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_vulndb'), debug_module=True)

	def getAPI(self, vulndb_api=None, session_id=None):
		if not vulndb_api:
			return ht.Config.getAPIKey('vulndb_api', session_id)
		return vulndb_api

	def getSuggestions(self, fieldType, fieldName, vulndb_api=None, session_id=None):
		self.headers = {
			'X-VulDB-ApiKey' : self.getAPI(vulndb_api, session_id)
		}
		params = {
			'type' : fieldType,
			'fieldName' : fieldName
		}
		suggest_url = '{h}{u}'.format(h=self.url, u=self.options['suggest'])
		response = requests.get(suggest_url, headers=self.headers, params=params)
		return response.text

	def getCVEInfo(self, CVE, references=False, vulndb_api=None, session_id=None):
		self.headers = {
			'X-VulDB-ApiKey' : self.getAPI(vulndb_api, session_id)
		}
		params = {
			'id' : CVE,
			'references' : references
		}
		suggest_url = '{h}{u}'.format(h=self.url, u=self.options['search-cve'])
		response = requests.get(suggest_url, headers=self.headers, params=params)
		return response.text

	